@@ -1,3 +1,5 @@
+{-# LANGUAGE FlexibleContexts #-}
+
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  XMonad.Hooks.DynamicLog
@@ -23,19 +25,22 @@ module XMonad.Hooks.DynamicLog (
     -- * Drop-in loggers
     dzen,
     xmobar,
+    statusBar,
     dynamicLog,
-    dynamicLogDzen,
-    dynamicLogXmobar,
     dynamicLogXinerama,
 
     -- * Build your own formatter
     dynamicLogWithPP,
     dynamicLogString,
-    PP(..), defaultPP, dzenPP, xmobarPP, sjanssenPP, byorgeyPP,
+    PP(..), defaultPP,
+
+    -- * Example formatters
+    dzenPP, xmobarPP, sjanssenPP, byorgeyPP,
 
     -- * Formatting utilities
-    wrap, pad, shorten,
-    xmobarColor, dzenColor, dzenEscape,
+    wrap, pad, trim, shorten,
+    xmobarColor, xmobarStrip,
+    dzenColor, dzenEscape, dzenStrip,
 
     -- * Internal formatting functions
     pprWindowSet,
@@ -50,6 +55,8 @@ module XMonad.Hooks.DynamicLog (
 -- Useful imports
 --
 import XMonad
+import Control.Monad
+import Data.Char ( isSpace )
 import Data.Maybe ( isJust, catMaybes )
 import Data.List
 import qualified Data.Map as M
@@ -72,21 +79,19 @@ import XMonad.Hooks.ManageDocks
 -- >    import XMonad.Hooks.DynamicLog
 --
 -- If you just want a quick-and-dirty status bar with zero effort, try
--- the 'dzen' function, which sets up a dzen status bar with a default
--- format:
+-- the 'xmobar' or 'dzen' functions:
 --
--- > main = dzen xmonad
+-- > main = xmonad =<< xmobar myConfig
+-- >
+-- > myConfig = defaultConfig { ... }
 --
--- or, to use this with your own custom xmonad configuration,
+-- There is also 'statusBar' if you'd like to use another status bar, or would
+-- like to use different formatting options.  The 'xmobar', 'dzen', and
+-- 'statusBar' functions are preferred over the other options listed below, as
+-- they take care of all the necessary plumbing -- no shell scripting required!
 --
--- > main = dzen $ \conf -> xmonad $ conf { <your customizations> }
---
--- Also you can use 'xmobar' function instead of 'dzen' in the examples above,
--- if you have xmobar installed.
---
--- Alternatively, you can choose among several default status bar
--- formats ('dynamicLog', 'dynamicLogDzen', 'dynamicLogXmobar', or
--- 'dynamicLogXinerama') by simply setting your logHook to the
+-- Alternatively, you can choose among several default status bar formats
+-- ('dynamicLog' or 'dynamicLogXinerama') by simply setting your logHook to the
 -- appropriate function, for instance:
 --
 -- > main = xmonad $ defaultConfig {
@@ -139,69 +144,71 @@ import XMonad.Hooks.ManageDocks
 
 ------------------------------------------------------------------------
 
--- | Run xmonad with a dzen status bar set to some nice defaults. Output
--- is taken from the dynamicLogWithPP hook.
+-- | Run xmonad with a dzen status bar set to some nice defaults.
 --
--- > main = dzen xmonad
+-- > main = xmonad =<< dzen myConfig
+-- >
+-- > myConfig = defaultConfig { ... }
 --
 -- The intent is that the above config file should provide a nice
--- status bar with minimal effort.  If you want to customize your xmonad
--- configuration while using this, you'll have to do something like
---
--- > main = dzen $ \conf -> xmonad $ conf { <your customized settings...> }
+-- status bar with minimal effort.
 --
 -- If you wish to customize the status bar format at all, you'll have to
--- use something like 'dynamicLogWithPP' instead.
+-- use the 'statusBar' function instead.
 --
 -- The binding uses the XMonad.Hooks.ManageDocks module to automatically
 -- handle screen placement for dzen, and enables 'mod-b' for toggling
 -- the menu bar.
 --
-dzen ::
-    (XConfig
-       (ModifiedLayout AvoidStruts
-          (Choose Tall (Choose (Mirror Tall) Full))) -> IO t) -> IO t
-dzen f = do
-  h <- spawnPipe ("dzen2" ++ " " ++ flags)
-  f $ defaultConfig
-           { logHook   = dynamicLogWithPP dzenPP
-                          { ppOutput = hPutStrLn h }
-           ,layoutHook = avoidStrutsOn [U] (layoutHook defaultConfig)
-           ,keys       = \c -> toggleStrutsKey c `M.union` keys defaultConfig c
-           ,manageHook = manageHook defaultConfig <+> manageDocks
-           }
+dzen :: LayoutClass l Window
+     => XConfig l -> IO (XConfig (ModifiedLayout AvoidStruts l))
+dzen conf = statusBar ("dzen2 " ++ flags) dzenPP toggleStrutsKey conf
  where
     fg      = "'#a8a3f7'" -- n.b quoting
     bg      = "'#3f3c6d'"
     flags   = "-e 'onstart=lower' -w 400 -ta l -fg " ++ fg ++ " -bg " ++ bg
 
 
--- | Run xmonad with a xmobar status bar set to some nice defaults. Output
--- is taken from the dynamicLogWithPP hook.
+-- | Run xmonad with a xmobar status bar set to some nice defaults.
 --
--- > main = xmobar xmonad
+-- > main = xmonad =<< xmobar myConfig
+-- >
+-- > myConfig = defaultConfig { ... }
 --
--- This works pretty much the same as 'dzen' function above
+-- This works pretty much the same as 'dzen' function above.
 --
-xmobar ::
-    (XConfig
-       (ModifiedLayout AvoidStruts
-          (Choose Tall (Choose (Mirror Tall) Full))) -> IO t) -> IO t
-xmobar f = do
-  h <- spawnPipe "xmobar"
-  f $ defaultConfig
-           { logHook    = dynamicLogWithPP xmobarPP { ppOutput = hPutStrLn h }
-           , layoutHook = avoidStruts $ layoutHook defaultConfig
-           , keys       = \c -> toggleStrutsKey c `M.union` keys defaultConfig c
-           , manageHook = manageHook defaultConfig <+> manageDocks
-           }
+xmobar :: LayoutClass l Window
+       => XConfig l -> IO (XConfig (ModifiedLayout AvoidStruts l))
+xmobar conf = statusBar "xmobar" xmobarPP toggleStrutsKey conf
+
+-- | Modifies the given base configuration to launch the given status bar,
+-- send status information to that bar, and allocate space on the screen edges
+-- for the bar.
+statusBar :: LayoutClass l Window
+          => String    -- ^ the command line to launch the status bar
+          -> PP        -- ^ the pretty printing options
+          -> (XConfig Layout -> (KeyMask, KeySym))
+                       -- ^ the desired key binding to toggle bar visibility
+          -> XConfig l -- ^ the base config
+          -> IO (XConfig (ModifiedLayout AvoidStruts l))
+statusBar cmd pp k conf = do
+    h <- spawnPipe cmd
+    return $ conf
+        { layoutHook = avoidStruts (layoutHook conf)
+        , logHook = do
+                        logHook conf
+                        dynamicLogWithPP pp { ppOutput = hPutStrLn h }
+        , manageHook = manageHook conf <+> manageDocks
+        , keys       = liftM2 M.union keys' (keys conf)
+        }
+ where
+    keys' = (`M.singleton` sendMessage ToggleStruts) . k
 
 -- |
 -- Helper function which provides ToggleStruts keybinding
 --
-toggleStrutsKey :: XConfig t -> M.Map (KeyMask, KeySym) (X ())
-toggleStrutsKey XConfig{modMask = modm} = M.fromList
-        [ ((modm, xK_b ), sendMessage ToggleStruts) ]
+toggleStrutsKey :: XConfig t -> (KeyMask, KeySym)
+toggleStrutsKey XConfig{modMask = modm} = (modm, xK_b )
 
 ------------------------------------------------------------------------
 
@@ -218,16 +225,6 @@ toggleStrutsKey XConfig{modMask = modm} = M.fromList
 dynamicLog :: X ()
 dynamicLog = dynamicLogWithPP defaultPP
 
--- | An example log hook that emulates dwm's status bar, using colour
--- codes printed to dzen.  Requires dzen. Workspaces, xinerama,
--- layouts and the window title are handled.
-dynamicLogDzen :: X ()
-dynamicLogDzen = dynamicLogWithPP dzenPP
-
--- | These are good defaults to be used with the xmobar status bar.
-dynamicLogXmobar :: X ()
-dynamicLogXmobar = dynamicLogWithPP xmobarPP
-
 -- | Format the current status using the supplied pretty-printing format,
 --   and write it to stdout.
 dynamicLogWithPP :: PP -> X ()
@@ -269,7 +266,7 @@ dynamicLogString pp = do
 pprWindowSet :: WorkspaceSort -> [Window] -> PP -> WindowSet -> String
 pprWindowSet sort' urgents pp s = sepBy (ppWsSep pp) . map fmt . sort' $
             map S.workspace (S.current s : S.visible s) ++ S.hidden s
-   where this     = S.tag (S.workspace (S.current s))
+   where this     = S.currentTag s
          visibles = map (S.tag . S.workspace) (S.visible s)
 
          fmt w = printer pp (S.tag w)
@@ -313,6 +310,11 @@ wrap l r m  = l ++ m ++ r
 pad :: String -> String
 pad = wrap " " " "
 
+-- | Trim leading and trailing whitespace from a string.
+trim :: String -> String
+trim = f . f
+    where f = reverse . dropWhile isSpace
+
 -- | Limit a string to a certain length, adding "..." if truncated.
 shorten :: Int -> String -> String
 shorten n xs | length xs < n = xs
@@ -343,6 +345,20 @@ dzenColor fg bg = wrap (fg1++bg1) (fg2++bg2)
 dzenEscape :: String -> String
 dzenEscape = concatMap (\x -> if x == '^' then "^^" else [x])
 
+-- | Strip dzen formatting or commands. Useful to remove ppHidden
+--   formatting in ppUrgent field. For example:
+--
+-- >     , ppHidden          = dzenColor "gray20" "" . wrap "(" ")"
+-- >     , ppUrgent          = dzenColor "dark orange" "" .  dzenStrip
+dzenStrip :: String -> String
+dzenStrip = strip [] where
+    strip keep x
+      | null x              = keep
+      | "^^" `isPrefixOf` x = strip (keep ++ "^") (drop 2 x)
+      | '^' == head x       = strip keep (drop 1 . dropWhile (/= ')') $ x)
+      | otherwise           = let (good,x') = span (/= '^') x
+                              in strip (keep ++ good) x'
+
 -- | Use xmobar escape codes to output a string with given foreground
 --   and background colors.
 xmobarColor :: String  -- ^ foreground color: a color name, or #rrggbb format
@@ -354,6 +370,21 @@ xmobarColor fg bg = wrap t "</fc>"
 
 -- ??? add an xmobarEscape function?
 
+-- | Strip xmobar markup. Useful to remove ppHidden color from ppUrgent
+--   field. For example:
+--
+-- >     , ppHidden          = xmobarColor "gray20" "" . wrap "<" ">"
+-- >     , ppUrgent          = xmobarColor "dark orange" "" .  xmobarStrip
+xmobarStrip :: String -> String
+xmobarStrip = strip [] where
+    strip keep x
+      | null x                 = keep
+      | "<fc="  `isPrefixOf` x = strip keep (drop 1 . dropWhile (/= '>') $ x)
+      | "</fc>" `isPrefixOf` x = strip keep (drop 5  x)
+      | '<' == head x          = strip (keep ++ "<") (tail x)
+      | otherwise              = let (good,x') = span (/= '<') x
+                                 in strip (keep ++ good) x'
+
 -- | The 'PP' type allows the user to customize the formatting of
 --   status information.
 data PP = PP { ppCurrent :: WorkspaceId -> String
@@ -426,13 +457,14 @@ defaultPP = PP { ppCurrent         = wrap "[" "]"
                , ppExtras          = []
                }
 
--- | Settings to emulate dwm's statusbar, dzen only.
+-- | Settings to emulate dwm's statusbar, dzen only. Uses dzenStrip in
+-- ppUrgent.
 dzenPP :: PP
 dzenPP = defaultPP { ppCurrent  = dzenColor "white" "#2b4f98" . pad
                      , ppVisible  = dzenColor "black" "#999999" . pad
                      , ppHidden   = dzenColor "black" "#cccccc" . pad
                      , ppHiddenNoWindows = const ""
-                     , ppUrgent   = dzenColor "red" "yellow"
+                     , ppUrgent   = dzenColor "red" "yellow" . dzenStrip
                      , ppWsSep    = ""
                      , ppSep      = ""
                      , ppLayout   = dzenColor "black" "#cccccc" .
@@ -450,6 +482,7 @@ xmobarPP :: PP
 xmobarPP = defaultPP { ppCurrent = xmobarColor "yellow" "" . wrap "[" "]"
                      , ppTitle   = xmobarColor "green"  "" . shorten 40
                      , ppVisible = wrap "(" ")"
+                     , ppUrgent = xmobarColor "red" "yellow"
                      }
 
 -- | The options that sjanssen likes to use with xmobar, as an
