@@ -1,5 +1,4 @@
-{-# OPTIONS_GHC -fglasgow-exts #-} -- for deriving Typeable
-{-# LANGUAGE FlexibleContexts, FlexibleInstances, MultiParamTypeClasses, PatternGuards, TypeSynonymInstances #-}
+{-# LANGUAGE DeriveDataTypeable, FlexibleContexts, FlexibleInstances, MultiParamTypeClasses, PatternGuards, TypeSynonymInstances #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  XMonad.Layout.Decoration
@@ -24,8 +23,9 @@ module XMonad.Layout.Decoration
     , DecorationStyle (..)
     , DefaultDecoration (..)
     , Shrinker (..), DefaultShrinker
-    , shrinkText, CustomShrink ( CustomShrink )
+    , shrinkText, CustomShrink ( CustomShrink ), shrinkWhile
     , isInStack, isVisible, isInvisible, isWithin, fi
+    , findWindowByDecoration
     , module XMonad.Layout.LayoutModifier
     ) where
 
@@ -201,7 +201,12 @@ instance Eq a => DecorationStyle DefaultDecoration a
 -- 'handleEvent', which will call the appropriate 'DecorationStyle'
 -- methods to perform its tasks.
 instance (DecorationStyle ds Window, Shrinker s) => LayoutModifier (Decoration ds s) Window where
-    redoLayout (Decoration st sh t ds) sc stack wrs
+    redoLayout (Decoration (I (Just s)) sh t ds) _ Nothing _ = do
+        releaseResources s
+        return ([], Just $ Decoration (I Nothing) sh t ds)
+    redoLayout _                                 _ Nothing _  = return ([], Nothing)
+
+    redoLayout (Decoration st sh t ds) sc (Just stack) wrs
         | I Nothing  <- st = initState t ds sc stack wrs >>= processState
         | I (Just s) <- st = do let dwrs  = decos s
                                     (d,a) = curry diff (get_ws dwrs) ws
@@ -264,11 +269,6 @@ instance (DecorationStyle ds Window, Shrinker s) => LayoutModifier (Decoration d
                                                       return $ Just $ Decoration (I Nothing) sh t  ds
     handleMess _ _ = return Nothing
 
-    emptyLayoutMod (Decoration (I (Just s)) sh t ds) _ _ = do
-        releaseResources s
-        return ([], Just $ Decoration (I Nothing) sh t ds)
-    emptyLayoutMod _ _ _  = return ([], Nothing)
-
     modifierDescription (Decoration _ _ _ ds) = describeDeco ds
 
 -- | By default 'Decoration' handles 'PropertyEvent' and 'ExposeEvent'
@@ -307,6 +307,9 @@ lookFor w ((wr,(Just dw,dr)):dwrs) | w == dw = Just (wr,(dw,dr))
 lookFor w ((_, (Nothing, _)):dwrs) = lookFor w dwrs
 lookFor _ [] = Nothing
 
+findWindowByDecoration :: Window -> DecorationState -> Maybe (OrigWin,(Window,Maybe Rectangle))
+findWindowByDecoration w ds = lookFor w (decos ds)
+
 -- | Initialize the 'DecorationState' by initializing the font
 -- structure and by creating the needed decorations.
 initState :: DecorationStyle ds Window => Theme -> ds Window -> Rectangle
