@@ -1,5 +1,4 @@
-{-# OPTIONS_GHC -fglasgow-exts #-} -- For deriving Data/Typeable
-{-# LANGUAGE FlexibleInstances, MultiParamTypeClasses, PatternGuards #-}
+{-# LANGUAGE DeriveDataTypeable, FlexibleInstances, MultiParamTypeClasses, PatternGuards #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module       : XMonad.Layout.LayoutCombinators
@@ -47,7 +46,7 @@ module XMonad.Layout.LayoutCombinators
       -- * New layout choice combinator and 'JumpToLayout'
       -- $jtl
     , (|||)
-    , JumpToLayout(JumpToLayout)
+    , JumpToLayout(..)
     ) where
 
 import Data.Maybe ( isJust, isNothing )
@@ -65,8 +64,8 @@ import XMonad.Layout.DragPane
 -- Then edit your @layoutHook@ to use the new layout combinators. For
 -- example:
 --
--- > myLayouts = (Tall 1 (3/100) (1/2) *//* Full)  ||| (Tall 1 (3/100) (1/2) ***||** Full) ||| Full ||| etc..
--- > main = xmonad defaultConfig { layoutHook = myLayouts }
+-- > myLayout = (Tall 1 (3/100) (1/2) *//* Full)  ||| (Tall 1 (3/100) (1/2) ***||** Full) ||| Full ||| etc..
+-- > main = xmonad defaultConfig { layoutHook = myLayout }
 --
 -- For more detailed instructions on editing the @layoutHook@ see:
 --
@@ -79,7 +78,7 @@ import XMonad.Layout.DragPane
 --
 -- Then bind some keys to a 'JumpToLayout' message:
 --
--- >   , ((modMask x .|. controlMask, xK_f), sendMessage $ JumpToLayout "Full")  -- jump directly to the Full layout
+-- >   , ((modm .|. controlMask, xK_f), sendMessage $ JumpToLayout "Full")  -- jump directly to the Full layout
 --
 -- See below for more detailed documentation.
 
@@ -214,12 +213,12 @@ infixr 5 |||
 
 data NewSelect l1 l2 a = NewSelect Bool (l1 a) (l2 a) deriving ( Read, Show )
 
-data NoWrap = NextLayoutNoWrap | Wrap deriving ( Read, Show, Typeable )
-instance Message NoWrap
-
--- | A message to jump to a particular layout, specified by its
---   description string.
-data JumpToLayout = JumpToLayout String deriving ( Read, Show, Typeable )
+-- | 
+data JumpToLayout = JumpToLayout String -- ^ A message to jump to a particular layout
+                                        -- , specified by its description string..
+                  | NextLayoutNoWrap
+                  | Wrap
+                    deriving ( Read, Show, Typeable )
 instance Message JumpToLayout
 
 instance (LayoutClass l1 a, LayoutClass l2 a) => LayoutClass (NewSelect l1 l2) a where
