@@ -1,8 +1,4 @@
-{-# OPTIONS_GHC -fglasgow-exts #-} -- for deriving Typeable
-{-# LANGUAGE PatternGuards           #-}
-{-# LANGUAGE FlexibleInstances       #-}
-{-# LANGUAGE MultiParamTypeClasses   #-}
-{-# LANGUAGE TypeSynonymInstances    #-}
+{-# LANGUAGE DeriveDataTypeable, PatternGuards, FlexibleInstances, MultiParamTypeClasses, TypeSynonymInstances    #-}
 -----------------------------------------------------------------------------
 -- |
 -- Module      :  XMonad.Layout.WindowArranger
@@ -57,20 +53,20 @@ import Data.Maybe
 -- You may also want to define some key binding to move or resize
 -- windows. These are good defaults:
 --
--- >        , ((modMask x .|. controlMask              , xK_s    ), sendMessage  Arrange         )
--- >        , ((modMask x .|. controlMask .|. shiftMask, xK_s    ), sendMessage  DeArrange       )
--- >        , ((modMask x .|. controlMask              , xK_Left ), sendMessage (MoveLeft      1))
--- >        , ((modMask x .|. controlMask              , xK_Right), sendMessage (MoveRight     1))
--- >        , ((modMask x .|. controlMask              , xK_Down ), sendMessage (MoveDown      1))
--- >        , ((modMask x .|. controlMask              , xK_Up   ), sendMessage (MoveUp        1))
--- >        , ((modMask x                 .|. shiftMask, xK_Left ), sendMessage (IncreaseLeft  1))
--- >        , ((modMask x                 .|. shiftMask, xK_Right), sendMessage (IncreaseRight 1))
--- >        , ((modMask x                 .|. shiftMask, xK_Down ), sendMessage (IncreaseDown  1))
--- >        , ((modMask x                 .|. shiftMask, xK_Up   ), sendMessage (IncreaseUp    1))
--- >        , ((modMask x .|. controlMask .|. shiftMask, xK_Left ), sendMessage (DecreaseLeft  1))
--- >        , ((modMask x .|. controlMask .|. shiftMask, xK_Right), sendMessage (DecreaseRight 1))
--- >        , ((modMask x .|. controlMask .|. shiftMask, xK_Down ), sendMessage (DecreaseDown  1))
--- >        , ((modMask x .|. controlMask .|. shiftMask, xK_Up   ), sendMessage (DecreaseUp    1))
+-- >        , ((modm .|. controlMask              , xK_s    ), sendMessage  Arrange         )
+-- >        , ((modm .|. controlMask .|. shiftMask, xK_s    ), sendMessage  DeArrange       )
+-- >        , ((modm .|. controlMask              , xK_Left ), sendMessage (MoveLeft      1))
+-- >        , ((modm .|. controlMask              , xK_Right), sendMessage (MoveRight     1))
+-- >        , ((modm .|. controlMask              , xK_Down ), sendMessage (MoveDown      1))
+-- >        , ((modm .|. controlMask              , xK_Up   ), sendMessage (MoveUp        1))
+-- >        , ((modm                 .|. shiftMask, xK_Left ), sendMessage (IncreaseLeft  1))
+-- >        , ((modm                 .|. shiftMask, xK_Right), sendMessage (IncreaseRight 1))
+-- >        , ((modm                 .|. shiftMask, xK_Down ), sendMessage (IncreaseDown  1))
+-- >        , ((modm                 .|. shiftMask, xK_Up   ), sendMessage (IncreaseUp    1))
+-- >        , ((modm .|. controlMask .|. shiftMask, xK_Left ), sendMessage (DecreaseLeft  1))
+-- >        , ((modm .|. controlMask .|. shiftMask, xK_Right), sendMessage (DecreaseRight 1))
+-- >        , ((modm .|. controlMask .|. shiftMask, xK_Down ), sendMessage (DecreaseDown  1))
+-- >        , ((modm .|. controlMask .|. shiftMask, xK_Up   ), sendMessage (DecreaseUp    1))
 --
 -- For detailed instructions on editing your key bindings, see
 -- "XMonad.Doc.Extending#Editing_key_bindings".
@@ -109,9 +105,9 @@ type ArrangeAll = Bool
 data WindowArranger a = WA Bool ArrangeAll [ArrangedWindow a] deriving (Read, Show)
 
 instance (Show a, Read a, Eq a) => LayoutModifier WindowArranger a where
-    pureModifier (WA True b   []) _  _              wrs = arrangeWindows b wrs
+    pureModifier (WA True b   []) _ (Just _)               wrs = arrangeWindows b wrs
 
-    pureModifier (WA True b awrs) _ (S.Stack w _ _) wrs = curry process wrs awrs
+    pureModifier (WA True b awrs) _ (Just (S.Stack w _ _)) wrs = curry process wrs awrs
         where
           wins         = map fst       *** map awrWin
           update (a,r) = mkNewAWRs b a *** removeAWRs r >>> uncurry (++)
@@ -203,7 +199,7 @@ memberFromList :: (b -> c) -> (c -> a -> Bool) -> a -> [b] -> [b]
 memberFromList f g l = foldr (h l) []
     where h x y ys = if g (f y) x then [y] else ys
 
--- | Get the list of elements to be deleted and the list ef elements to
+-- | Get the list of elements to be deleted and the list of elements to
 -- be added to the first list in order to get the second list.
 diff :: Eq a => ([a],[a]) -> ([a],[a])
 diff (x,y) = (x \\ y, y \\ x)
