@@ -1,5 +1,4 @@
-{-# OPTIONS_GHC -fglasgow-exts #-} -- For deriving Data/Typeable
-{-# LANGUAGE FlexibleInstances, GeneralizedNewtypeDeriving, MultiParamTypeClasses, TypeSynonymInstances #-}
+{-# LANGUAGE DeriveDataTypeable, FlexibleInstances, GeneralizedNewtypeDeriving, MultiParamTypeClasses, TypeSynonymInstances, PatternGuards #-}
 
 -----------------------------------------------------------------------------
 -- |
@@ -19,7 +18,7 @@ module XMonad.Layout.WindowNavigation (
                                    -- * Usage
                                    -- $usage
                                    windowNavigation, configurableNavigation,
-                                   Navigate(..), Direction(..),
+                                   Navigate(..), Direction2D(..),
                                    MoveWindowToWindow(..),
                                    navigateColor, navigateBrightness,
                                    noNavigateBorders, defaultWNConfig
@@ -30,10 +29,9 @@ import XMonad hiding (Point)
 import qualified XMonad.StackSet as W
 import XMonad.Layout.LayoutModifier
 import XMonad.Util.Invisible
+import XMonad.Util.Types (Direction2D(..))
 import XMonad.Util.XUtils
 
-import XMonad.Hooks.ManageDocks (Direction(..))
-
 -- $usage
 -- You can use this module with the following in your @~\/.xmonad\/xmonad.hs@:
 --
@@ -42,8 +40,8 @@ import XMonad.Hooks.ManageDocks (Direction(..))
 -- Then edit your @layoutHook@ by adding the WindowNavigation layout modifier
 -- to some layout:
 --
--- > myLayouts = windowNavigation (Tall 1 (3/100) (1/2))  ||| Full ||| etc..
--- > main = xmonad defaultConfig { layoutHook = myLayouts }
+-- > myLayout = windowNavigation (Tall 1 (3/100) (1/2))  ||| Full ||| etc..
+-- > main = xmonad defaultConfig { layoutHook = myLayout }
 --
 -- For more detailed instructions on editing the layoutHook see:
 --
@@ -51,14 +49,14 @@ import XMonad.Hooks.ManageDocks (Direction(..))
 --
 -- In keybindings:
 --
--- >    , ((modMask x,                 xK_Right), sendMessage $ Go R)
--- >    , ((modMask x,                 xK_Left ), sendMessage $ Go L)
--- >    , ((modMask x,                 xK_Up   ), sendMessage $ Go U)
--- >    , ((modMask x,                 xK_Down ), sendMessage $ Go D)
--- >    , ((modMask x .|. controlMask, xK_Right), sendMessage $ Swap R)
--- >    , ((modMask x .|. controlMask, xK_Left ), sendMessage $ Swap L)
--- >    , ((modMask x .|. controlMask, xK_Up   ), sendMessage $ Swap U)
--- >    , ((modMask x .|. controlMask, xK_Down ), sendMessage $ Swap D)
+-- >    , ((modm,                 xK_Right), sendMessage $ Go R)
+-- >    , ((modm,                 xK_Left ), sendMessage $ Go L)
+-- >    , ((modm,                 xK_Up   ), sendMessage $ Go U)
+-- >    , ((modm,                 xK_Down ), sendMessage $ Go D)
+-- >    , ((modm .|. controlMask, xK_Right), sendMessage $ Swap R)
+-- >    , ((modm .|. controlMask, xK_Left ), sendMessage $ Swap L)
+-- >    , ((modm .|. controlMask, xK_Up   ), sendMessage $ Swap U)
+-- >    , ((modm .|. controlMask, xK_Down ), sendMessage $ Swap D)
 --
 -- For detailed instruction on editing the key binding see:
 --
@@ -68,7 +66,9 @@ import XMonad.Hooks.ManageDocks (Direction(..))
 data MoveWindowToWindow a = MoveWindowToWindow a a deriving ( Read, Show, Typeable )
 instance Typeable a => Message (MoveWindowToWindow a)
 
-data Navigate = Go Direction | Swap Direction | Move Direction deriving ( Read, Show, Typeable )
+data Navigate = Go Direction2D | Swap Direction2D | Move Direction2D
+              | Apply (Window -> X()) Direction2D -- ^ Apply action with destination window
+        deriving ( Typeable )
 instance Message Navigate
 
 data WNConfig =
@@ -88,9 +88,7 @@ navigateColor c =
     WNC Nothing c c c c
 
 navigateBrightness :: Double -> WNConfig
-navigateBrightness f | f > 1 = navigateBrightness 1
-                     | f < 0 = navigateBrightness 0
-navigateBrightness f = defaultWNConfig { brightness = Just f }
+navigateBrightness f = defaultWNConfig { brightness = Just $ max 0 $ min 1 f }
 
 defaultWNConfig :: WNConfig
 defaultWNConfig = WNC (Just 0.4) "#0000FF" "#00FFFF" "#FF0000" "#FF00FF"
@@ -106,7 +104,7 @@ configurableNavigation :: LayoutClass l a => WNConfig -> l a -> ModifiedLayout W
 configurableNavigation conf = ModifiedLayout (WindowNavigation conf (I Nothing))
 
 instance LayoutModifier WindowNavigation Window where
-    redoLayout (WindowNavigation conf (I state)) rscr s origwrs =
+    redoLayout (WindowNavigation conf (I state)) rscr (Just s) origwrs =
         do XConf { normalBorder = nbc, focusedBorder = fbc, display = dpy } <- ask
            [uc,dc,lc,rc] <-
                case brightness conf of
@@ -127,15 +125,16 @@ instance LayoutModifier WindowNavigation Window where
                wrs = filter ((`elem` existing_wins) . fst) $ filter ((/=r) . snd) $
                      filter ((/=w) . fst) origwrs
                wnavigable = nub $ concatMap
-                            (\d -> truncHead $ navigable d pt wrs) [U,D,R,L]
+                            (\d -> take 1 $ navigable d pt wrs) [U,D,R,L]
                wnavigablec = nub $ concatMap
                             (\d -> map (\(win,_) -> (win,dirc d)) $
-                                   truncHead $ navigable d pt wrs) [U,D,R,L]
+                                   take 1 $ navigable d pt wrs) [U,D,R,L]
                wothers = case state of Just (NS _ wo) -> map fst wo
                                        _              -> []
            mapM_ (sc nbc) (wothers \\ map fst wnavigable)
            mapM_ (\(win,c) -> sc c win) wnavigablec
            return (origwrs, Just $ WindowNavigation conf $ I $ Just $ NS pt wnavigable)
+    redoLayout _ _ _ origwrs = return (origwrs, Nothing)
 
     handleMessOrMaybeModifyIt (WindowNavigation conf (I (Just (NS pt wrs)))) m
         | Just (Go d) <- fromMessage m =
@@ -176,6 +175,10 @@ instance LayoutModifier WindowNavigation Window where
              ((w,_):_) -> do mst <- gets (W.stack . W.workspace . W.current . windowset)
                              return $ do st <- mst
                                          Just $ Right $ SomeMessage $ MoveWindowToWindow (W.focus st) w
+        | Just (Apply f d) <- fromMessage m =
+             case navigable d pt wrs of
+             []         -> return Nothing
+             ((w,_):_)  -> f w >> return Nothing
         | Just Hide <- fromMessage m =
                        do XConf { normalBorder = nbc } <- ask
                           mapM_ (sc nbc . fst) wrs
@@ -184,24 +187,20 @@ instance LayoutModifier WindowNavigation Window where
                handleMessOrMaybeModifyIt (WindowNavigation conf (I $ Just (NS pt wrs))) (SomeMessage Hide)
     handleMessOrMaybeModifyIt _ _ = return Nothing
 
-navigable :: Direction -> Point -> [(Window, Rectangle)] -> [(Window, Rectangle)]
+navigable :: Direction2D -> Point -> [(Window, Rectangle)] -> [(Window, Rectangle)]
 navigable d pt = sortby d . filter (inr d pt . snd)
 
-truncHead :: [a] -> [a]
-truncHead (x:_) = [x]
-truncHead [] = []
-
 sc :: Pixel -> Window -> X ()
 sc c win = withDisplay $ \dpy -> io $ setWindowBorder dpy win c
 
 center :: Rectangle -> Point
 center (Rectangle x y w h) = P (fromIntegral x + fromIntegral w/2)  (fromIntegral y + fromIntegral h/2)
 
-centerd :: Direction -> Point -> Rectangle -> Point
+centerd :: Direction2D -> Point -> Rectangle -> Point
 centerd d (P xx yy) (Rectangle x y w h) | d == U || d == D = P xx (fromIntegral y + fromIntegral h/2)
                                         | otherwise = P (fromIntegral x + fromIntegral w/2) yy
 
-inr :: Direction -> Point -> Rectangle -> Bool
+inr :: Direction2D -> Point -> Rectangle -> Bool
 inr D (P x y) (Rectangle l yr w h) = x >= fromIntegral l && x < fromIntegral l + fromIntegral w &&
                                      y <  fromIntegral yr + fromIntegral h
 inr U (P x y) (Rectangle l yr w _) = x >= fromIntegral l && x < fromIntegral l + fromIntegral w &&
@@ -215,7 +214,7 @@ inrect :: Point -> Rectangle -> Bool
 inrect (P x y) (Rectangle a b w h) = x >  fromIntegral a && x < fromIntegral a + fromIntegral w &&
                                      y >  fromIntegral b && y < fromIntegral b + fromIntegral h
 
-sortby :: Direction -> [(a,Rectangle)] -> [(a,Rectangle)]
+sortby :: Direction2D -> [(a,Rectangle)] -> [(a,Rectangle)]
 sortby U = sortBy (\(_,Rectangle _ y _ _) (_,Rectangle _ y' _ _) -> compare y' y)
 sortby D = sortBy (\(_,Rectangle _ y _ _) (_,Rectangle _ y' _ _) -> compare y y')
 sortby R = sortBy (\(_,Rectangle x _ _ _) (_,Rectangle x' _ _ _) -> compare x x')
