@@ -1,5 +1,4 @@
-{-# OPTIONS_GHC -fglasgow-exts #-} -- For deriving Data/Typeable
-{-# LANGUAGE FlexibleInstances, MultiParamTypeClasses #-}
+{-# LANGUAGE DeriveDataTypeable, FlexibleInstances, MultiParamTypeClasses, TypeSynonymInstances, PatternGuards #-}
 
 -----------------------------------------------------------------------------
 -- |
@@ -37,7 +36,7 @@ import XMonad.Util.Run ( runProcessWithInput )
 import XMonad.Prompt ( XPConfig )
 import XMonad.Prompt.Directory ( directoryPrompt )
 import XMonad.Layout.LayoutModifier
-import XMonad.StackSet ( tag, current, workspace )
+import XMonad.StackSet ( tag, currentTag )
 
 -- $usage
 -- You can use this module with the following in your @~\/.xmonad\/xmonad.hs@:
@@ -47,8 +46,8 @@ import XMonad.StackSet ( tag, current, workspace )
 -- Then edit your @layoutHook@ by adding the Workspace layout modifier
 -- to some layout:
 --
--- > myLayouts = workspaceDir "~" (Tall 1 (3/100) (1/2))  ||| Full ||| etc..
--- > main = xmonad defaultConfig { layoutHook = myLayouts }
+-- > myLayout = workspaceDir "~" (Tall 1 (3/100) (1/2))  ||| Full ||| etc..
+-- > main = xmonad defaultConfig { layoutHook = myLayout }
 --
 -- For more detailed instructions on editing the layoutHook see:
 --
@@ -57,7 +56,7 @@ import XMonad.StackSet ( tag, current, workspace )
 -- WorkspaceDir provides also a prompt. To use it you need to import
 -- "XMonad.Prompt" and add something like this to your key bindings:
 --
--- >  , ((modMask x .|. shiftMask, xK_x     ), changeDir defaultXPConfig)
+-- >  , ((modm .|. shiftMask, xK_x     ), changeDir defaultXPConfig)
 --
 -- For detailed instruction on editing the key binding see:
 --
@@ -69,7 +68,7 @@ instance Message Chdir
 data WorkspaceDir a = WorkspaceDir String deriving ( Read, Show )
 
 instance LayoutModifier WorkspaceDir Window where
-    modifyLayout (WorkspaceDir d) w r = do tc <- gets (tag.workspace.current.windowset)
+    modifyLayout (WorkspaceDir d) w r = do tc <- gets (currentTag.windowset)
                                            when (tc == tag w) $ scd d
                                            runLayout w r
     handleMess (WorkspaceDir _) m
