@@ -5,7 +5,7 @@
 -- Copyright   :  (C) 2007 Andrea Rossato
 -- License     :  BSD3
 --
--- Maintainer  :  andrea.rossato@unibz.it
+-- Maintainer  :  Spencer Janssen <spencerjanssen@gmail.com>
 -- Stability   :  unstable
 -- Portability :  unportable
 --
@@ -18,11 +18,21 @@ module XMonad.Prompt
       -- $usage
       mkXPrompt
     , mkXPromptWithReturn
+    , amberXPConfig
     , defaultXPConfig
+    , greenXPConfig
     , XPType (..)
     , XPPosition (..)
     , XPConfig (..)
     , XPrompt (..)
+    , XP
+    , defaultXPKeymap
+    , quit
+    , killBefore, killAfter, startOfLine, endOfLine
+    , pasteString, copyString, moveCursor
+    , moveWord, killWord, deleteString
+    , moveHistory, setSuccess, setDone
+    , Direction1D(..)
     , ComplFunction
     -- * X Utilities
     -- $xutils
@@ -44,26 +54,37 @@ module XMonad.Prompt
     , decodeInput
     , encodeOutput
     , historyCompletion
+    , historyCompletionP
+    -- * History filters
+    , deleteAllDuplicates
+    , deleteConsecutive
     ) where
 
-import XMonad  hiding (config, io)
+import Prelude hiding (catch)
+
+import XMonad  hiding (config, io, numlockMask, cleanMask)
+import qualified XMonad as X (numlockMask,config)
 import qualified XMonad.StackSet as W
 import XMonad.Util.Font
-import XMonad.Util.XSelection (getSelection)
+import XMonad.Util.Types
+import XMonad.Util.XSelection (getSelection, putSelection)
 
-import Control.Arrow ((&&&))
+import Control.Arrow ((&&&),first)
 import Control.Concurrent (threadDelay)
 import Control.Monad.Reader
 import Control.Monad.State
 import Control.Applicative ((<$>))
 import Data.Char
-import Data.Bits ((.&.))
+import Data.Bits
 import Data.Maybe
 import Data.List
 import Data.Set (fromList, toList)
-import System.Environment (getEnv)
+import System.Directory
 import System.IO
 import System.Posix.Files
+import Control.Exception hiding (handle)
+
+import qualified Data.Map as M
 
 -- $usage
 -- For usage examples see "XMonad.Prompt.Shell",
@@ -83,13 +104,16 @@ data XPState =
         , complWin           :: Maybe Window
         , complWinDim        :: Maybe ComplWindowDim
         , completionFunction :: String -> IO [String]
+        , showComplWin       :: Bool
         , gcon               :: !GC
         , fontS              :: !XMonadFont
         , xptype             :: !XPType
-        , command            :: String
+        , commandHistory     :: W.Stack String
         , offset             :: !Int
-        , history            :: [History]
         , config             :: XPConfig
+        , successful         :: Bool
+        , numlockMask        :: KeyMask
+        , done               :: Bool
         }
 
 data XPConfig =
@@ -103,10 +127,17 @@ data XPConfig =
         , position          :: XPPosition -- ^ Position: 'Top' or 'Bottom'
         , height            :: !Dimension -- ^ Window height
         , historySize       :: !Int       -- ^ The number of history entries to be saved
+        , historyFilter     :: [String] -> [String]
+                                         -- ^ a filter to determine which
+                                         -- history entries to remember
+        , promptKeymap      :: M.Map (KeyMask,KeySym) (XP ())
+                                         -- ^ Mapping from key combinations to actions
+        , completionKey     :: KeySym     -- ^ Key that should trigger completion
         , defaultText       :: String     -- ^ The text by default in the prompt line
         , autoComplete      :: Maybe Int  -- ^ Just x: if only one completion remains, auto-select it,
-                                          --   and delay by x microseconds
-        } deriving (Show, Read)
+        , showCompletionOnTab :: Bool     -- ^ Only show list of completions when Tab was pressed
+                                         --   and delay by x microseconds
+        }
 
 data XPType = forall p . XPrompt p => XPT p
 
@@ -140,12 +171,12 @@ class XPrompt t where
     -- string presently in the command line and the list of
     -- completion.
     nextCompletion :: t -> String -> [String] -> String
-    nextCompletion t c l = getNextOfLastWord t c l
+    nextCompletion = getNextOfLastWord
 
     -- | This method is used to generate the string to be passed to
     -- the completion function.
     commandToComplete :: t -> String -> String
-    commandToComplete _ c = getLastWord c
+    commandToComplete _ = getLastWord
 
     -- | This method is used to process each completion in order to
     -- generate the string that will be compared with the command
@@ -161,26 +192,32 @@ data XPPosition = Top
                 | Bottom
                   deriving (Show,Read)
 
-defaultXPConfig :: XPConfig
+amberXPConfig, defaultXPConfig, greenXPConfig :: XPConfig
+
 defaultXPConfig =
-    XPC { font              = "-misc-fixed-*-*-*-*-10-*-*-*-*-*-*-*"
-        , bgColor           = "#333333"
-        , fgColor           = "#FFFFFF"
-        , fgHLight          = "#000000"
-        , bgHLight          = "#BBBBBB"
-        , borderColor       = "#FFFFFF"
+    XPC { font              = "-misc-fixed-*-*-*-*-12-*-*-*-*-*-*-*"
+        , bgColor           = "grey22"
+        , fgColor           = "grey80"
+        , fgHLight          = "black"
+        , bgHLight          = "grey"
+        , borderColor       = "white"
         , promptBorderWidth = 1
+        , promptKeymap      = defaultXPKeymap
+        , completionKey     = xK_Tab
         , position          = Bottom
         , height            = 18
         , historySize       = 256
+        , historyFilter     = id
         , defaultText       = []
         , autoComplete      = Nothing
-        }
+        , showCompletionOnTab = False }
+greenXPConfig = defaultXPConfig { fgColor = "green", bgColor = "black" }
+amberXPConfig = defaultXPConfig { fgColor = "#ca8f2d", bgColor = "black", fgHLight = "#eaaf4c" }
 
 type ComplFunction = String -> IO [String]
 
 initState :: XPrompt p => Display -> Window -> Window -> Rectangle -> ComplFunction
-          -> GC -> XMonadFont -> p -> [History] -> XPConfig -> XPState
+          -> GC -> XMonadFont -> p -> [String] -> XPConfig -> XPState
 initState d rw w s compl gc fonts pt h c =
     XPS { dpy                = d
         , rootw              = rw
@@ -189,15 +226,27 @@ initState d rw w s compl gc fonts pt h c =
         , complWin           = Nothing
         , complWinDim        = Nothing
         , completionFunction = compl
+        , showComplWin       = not (showCompletionOnTab c)
         , gcon               = gc
         , fontS              = fonts
         , xptype             = XPT pt
-        , command            = defaultText c
+        , commandHistory     = W.Stack { W.focus = defaultText c
+                                       , W.up    = []
+                                       , W.down  = h }
         , offset             = length (defaultText c)
-        , history            = h
         , config             = c
+        , successful         = False
+        , done               = False
+        , numlockMask        = X.numlockMask defaultConfig
         }
 
+-- this would be much easier with functional references
+command :: XPState -> String
+command = W.focus . commandHistory
+
+setCommand :: String -> XPState -> XPState
+setCommand xs s = s { commandHistory = (commandHistory s) { W.focus = xs }}
+
 -- | Same as 'mkXPrompt', except that the action function can have
 --   type @String -> X a@, for any @a@, and the final action returned
 --   by 'mkXPromptWithReturn' will have type @X (Maybe a)@.  @Nothing@
@@ -210,25 +259,32 @@ mkXPromptWithReturn t conf compl action = do
   let d = display c
       rw = theRoot c
   s <- gets $ screenRect . W.screenDetail . W.current . windowset
+  hist <- liftIO readHistory
   w <- liftIO $ createWin d rw conf s
   liftIO $ selectInput d w $ exposureMask .|. keyPressMask
   gc <- liftIO $ createGC d w
   liftIO $ setGraphicsExposures d gc False
-  (hist,h) <- liftIO $ readHistory
   fs <- initXMF (font conf)
-  let st = initState d rw w s compl gc fs (XPT t) hist conf
+  numlock <- asks $ X.numlockMask . X.config
+  let hs = fromMaybe [] $ M.lookup (showXPrompt t) hist
+      st = (initState d rw w s compl gc fs (XPT t) hs conf)
+           { numlockMask = numlock }
   st' <- liftIO $ execStateT runXP st
 
   releaseXMF fs
   liftIO $ freeGC d gc
-  liftIO $ hClose h
-  if (command st' /= "")
+  if successful st'
     then do
-      let htw = take (historySize conf) (history st')
-      liftIO $ writeHistory htw
+      liftIO $ writeHistory $ M.insertWith
+                                (\xs ys -> take (historySize conf)
+                                        . historyFilter conf $ xs ++ ys)
+                                (showXPrompt t) (historyFilter conf [command st'])
+                                hist
+                                -- we need to apply historyFilter before as well, since
+                                -- otherwise the filter would not be applied if
+                                -- there is no history
       Just <$> action (command st')
-    else
-      return Nothing
+    else return Nothing
 
 -- | Creates a prompt given:
 --
@@ -271,42 +327,54 @@ eventLoop action = do
                         else return (Nothing, "")
               return (ks,s,ev)
   action (fromMaybe xK_VoidSymbol keysym,string) event
+  gets done >>= flip unless (eventLoop handle)
+
+-- | Removes numlock and capslock from a keymask.
+-- Duplicate of cleanMask from core, but in the
+-- XP monad instead of X.
+cleanMask :: KeyMask -> XP KeyMask
+cleanMask msk = do
+  numlock <- gets numlockMask
+  let highMasks = 1 `shiftL` 12 - 1
+  return (complement (numlock .|. lockMask) .&. msk .&. highMasks)
 
 -- Main event handler
 handle :: KeyStroke -> Event -> XP ()
-handle k@(ks,_) e@(KeyEvent {ev_event_type = t})
-    | t == keyPress && ks == xK_Tab    = do
+handle ks@(sym,_) e@(KeyEvent {ev_event_type = t, ev_state = m}) = do
+  complKey <- gets $ completionKey . config
   c <- getCompletions
-  completionHandle c k e
-handle ks (KeyEvent {ev_event_type = t, ev_state = m})
-    | t == keyPress = keyPressHandle m ks
+  when (length c > 1) $ modify (\s -> s { showComplWin = True })
+  if complKey == sym
+     then completionHandle c ks e
+     else when (t == keyPress) $ keyPressHandle m ks
 handle _ (ExposeEvent {ev_window = w}) = do
   st <- get
   when (win st == w) updateWindows
-  eventLoop handle
-handle _  _ = eventLoop handle
+handle _  _ = return ()
 
 -- completion event handler
 completionHandle ::  [String] -> KeyStroke -> Event -> XP ()
-completionHandle c (ks,_) (KeyEvent {ev_event_type = t})
-    | t == keyPress && ks == xK_Tab = do
-  st <- get
-  let updateState l = do let new_command = nextCompletion (xptype st) (command st) l
-                         modify $ \s ->  s { command = new_command, offset = length new_command }
-      updateWins  l = do redrawWindows l
-                         eventLoop (completionHandle l)
-  case c of
-    []  -> updateWindows   >> eventLoop handle
-    [x] -> updateState [x] >> getCompletions >>= updateWins
-    l   -> updateState l   >> updateWins l
--- key release
-    | t == keyRelease && ks == xK_Tab = eventLoop (completionHandle c)
--- other keys
-completionHandle _ ks (KeyEvent {ev_event_type = t, ev_state = m})
-    | t == keyPress = keyPressHandle m ks
+completionHandle c ks@(sym,_) (KeyEvent { ev_event_type = t, ev_state = m }) = do
+  complKey <- gets $ completionKey . config
+  case () of
+    () | t == keyPress && sym == complKey ->
+          do
+            st <- get
+            let updateState l =
+                    let new_command = nextCompletion (xptype st) (command st) l
+                    in modify $ \s -> setCommand new_command $ s { offset = length new_command }
+                updateWins  l = redrawWindows l >>
+                                eventLoop (completionHandle l)
+            case c of
+              []  -> updateWindows   >> eventLoop handle
+              [x] -> updateState [x] >> getCompletions >>= updateWins
+              l   -> updateState l   >> updateWins l
+      | t == keyRelease && sym == complKey -> eventLoop (completionHandle c)
+      | otherwise -> keyPressHandle m ks -- some other key, handle it normally
 -- some other event: go back to main loop
 completionHandle _ k e = handle k e
 
+
 tryAutoComplete :: XP Bool
 tryAutoComplete = do
     ac <- gets (autoComplete . config)
@@ -319,73 +387,85 @@ tryAutoComplete = do
   where runCompleted cmd delay = do
             st <- get
             let new_command = nextCompletion (xptype st) (command st) [cmd]
-            modify $ \s -> s { command = "autocompleting..." }
+            modify $ setCommand "autocompleting..."
             updateWindows
             io $ threadDelay delay
-            modify $ \s -> s { command = new_command }
-            historyPush
+            modify $ setCommand new_command
             return True
 
 -- KeyPresses
 
-data Direction = Prev | Next deriving (Eq,Show,Read)
+defaultXPKeymap :: M.Map (KeyMask,KeySym) (XP ())
+defaultXPKeymap = M.fromList $
+  map (first $ (,) controlMask) -- control + <key>
+  [ (xK_u, killBefore)
+  , (xK_k, killAfter)
+  , (xK_a, startOfLine)
+  , (xK_e, endOfLine)
+  , (xK_y, pasteString)
+  , (xK_c, copyString)
+  , (xK_Right, moveWord Next)
+  , (xK_Left, moveWord Prev)
+  , (xK_Delete, killWord Next)
+  , (xK_BackSpace, killWord Prev)
+  , (xK_w, killWord Prev)
+  , (xK_q, quit)
+  ] ++
+  map (first $ (,) 0)
+  [ (xK_Return, setSuccess True >> setDone True)
+  , (xK_KP_Enter, setSuccess True >> setDone True)
+  , (xK_BackSpace, deleteString Prev)
+  , (xK_Delete, deleteString Next)
+  , (xK_Left, moveCursor Prev)
+  , (xK_Right, moveCursor Next)
+  , (xK_Home, startOfLine)
+  , (xK_End, endOfLine)
+  , (xK_Down, moveHistory W.focusUp')
+  , (xK_Up, moveHistory W.focusDown')
+  , (xK_Escape, quit)
+  ]
 
 keyPressHandle :: KeyMask -> KeyStroke -> XP ()
--- commands: ctrl + ... todo
-keyPressHandle mask (ks,_)
-    | (mask .&. controlMask) > 0 =
-        -- control sequences
-        case () of
-          _ | ks == xK_u               -> killBefore    >> go
-            | ks == xK_k               -> killAfter     >> go
-            | ks == xK_a               -> startOfLine   >> go
-            | ks == xK_e               -> endOfLine     >> go
-            | ks == xK_y               -> pasteString   >> go
-            | ks == xK_Right           -> moveWord Next >> go
-            | ks == xK_Left            -> moveWord Prev >> go
-            | ks == xK_Delete          -> killWord Next >> go
-            | ks == xK_BackSpace       -> killWord Prev >> go
-            | ks == xK_w               -> killWord Prev >> go
-            | ks == xK_g || ks == xK_c -> quit
-            | otherwise  -> eventLoop handle -- unhandled control sequence
-    | ks == xK_Return    = historyPush       >> return ()
-    | ks == xK_BackSpace = deleteString Prev >> go
-    | ks == xK_Delete    = deleteString Next >> go
-    | ks == xK_Left      = moveCursor   Prev >> go
-    | ks == xK_Right     = moveCursor   Next >> go
-    | ks == xK_Up        = moveHistory  Prev >> go
-    | ks == xK_Down      = moveHistory  Next >> go
-    | ks == xK_Home      = startOfLine       >> go
-    | ks == xK_End       = endOfLine         >> go
-    | ks == xK_Escape    = quit
-    where
-      go   = updateWindows >> eventLoop handle
-      quit = flushString   >> return () -- quit and discard everything
--- insert a character
-keyPressHandle _ (_,s)
-    | s == "" = eventLoop handle
-    | otherwise = do insertString (decodeInput s)
-                     updateWindows
-                     completed <- tryAutoComplete
-                     unless completed $ eventLoop handle
+keyPressHandle m (ks,str) = do
+  km <- gets (promptKeymap . config)
+  mask <- cleanMask m
+  case M.lookup (mask,ks) km of
+    Just action -> action >> updateWindows
+    Nothing -> case str of
+                 "" -> eventLoop handle
+                 _ -> when (mask .&. controlMask == 0) $ do
+                                 insertString (decodeInput str)
+                                 updateWindows
+                                 completed <- tryAutoComplete
+                                 when completed $ setSuccess True >> setDone True
+
+setSuccess :: Bool -> XP ()
+setSuccess b = modify $ \s -> s { successful = b }
+
+setDone :: Bool -> XP ()
+setDone b = modify $ \s -> s { done = b }
 
 -- KeyPress and State
 
+-- | Quit.
+quit :: XP ()
+quit = flushString >> setSuccess False >> setDone True
+
 -- | Kill the portion of the command before the cursor
 killBefore :: XP ()
 killBefore =
-  modify $ \s -> s { command = drop (offset s) (command s)
-                   , offset  = 0 }
+  modify $ \s -> setCommand (drop (offset s) (command s)) $ s { offset  = 0 }
 
 -- | Kill the portion of the command including and after the cursor
 killAfter :: XP ()
 killAfter =
-  modify $ \s -> s { command = take (offset s) (command s) }
+  modify $ \s -> setCommand (take (offset s) (command s)) s
 
 -- | Kill the next\/previous word
-killWord :: Direction -> XP ()
+killWord :: Direction1D -> XP ()
 killWord d = do
-  XPS { command = c, offset = o } <- get
+  o <- gets offset
+  c <- gets command
   let (f,ss)        = splitAt o c
       delNextWord w =
           case w of
@@ -396,7 +476,7 @@ killWord d = do
           case d of
             Next -> (f ++ delNextWord ss, o)
             Prev -> (delPrevWord f ++ ss, length $ delPrevWord f) -- laziness!!
-  modify $ \s -> s { command = ncom, offset = noff}
+  modify $ \s -> setCommand ncom $ s { offset = noff}
 
 -- | Put the cursor at the end of line
 endOfLine :: XP ()
@@ -410,13 +490,12 @@ startOfLine  =
 
 -- |  Flush the command string and reset the offset
 flushString :: XP ()
-flushString = do
-  modify $ \s -> s { command = "", offset = 0}
+flushString = modify $ \s -> setCommand "" $ s { offset = 0}
 
 -- | Insert a character at the cursor position
 insertString :: String -> XP ()
 insertString str =
-  modify $ \s -> s { command = c (command s) (offset s), offset = o (offset s)}
+  modify $ \s -> setCommand (c (command s) (offset s)) $ s { offset = o (offset s)}
   where o oo = oo + length str
         c oc oo | oo >= length oc = oc ++ str
                 | otherwise = f ++ str ++ ss
@@ -424,12 +503,16 @@ insertString str =
 
 -- | Insert the current X selection string at the cursor position.
 pasteString :: XP ()
-pasteString = join $ io $ liftM insertString $ getSelection
+pasteString = join $ io $ liftM insertString getSelection
+
+-- | Copy the currently entered string into the X selection.
+copyString :: XP ()
+copyString = gets command >>= io . putSelection
 
 -- | Remove a character at the cursor position
-deleteString :: Direction -> XP ()
+deleteString :: Direction1D -> XP ()
 deleteString d =
-  modify $ \s -> s { command = c (command s) (offset s), offset = o (offset s)}
+  modify $ \s -> setCommand (c (command s) (offset s)) $ s { offset = o (offset s)}
   where o oo = if d == Prev then max 0 (oo - 1) else oo
         c oc oo
             | oo >= length oc && d == Prev = take (oo - 1) oc
@@ -439,13 +522,13 @@ deleteString d =
             where (f,ss) = splitAt oo oc
 
 -- | move the cursor one position
-moveCursor :: Direction -> XP ()
+moveCursor :: Direction1D -> XP ()
 moveCursor d =
   modify $ \s -> s { offset = o (offset s) (command s)}
   where o oo c = if d == Prev then max 0 (oo - 1) else min (length c) (oo + 1)
 
 -- | move the cursor one word
-moveWord :: Direction -> XP ()
+moveWord :: Direction1D -> XP ()
 moveWord d = do
   c <- gets command
   o <- gets offset
@@ -455,21 +538,14 @@ moveWord d = do
                  ' ':x -> 1 + lenToS x
                  x     -> lenToS x
       newoff = case d of
-                 Prev -> o - (ln reverse f )
-                 Next -> o + (ln id      ss)
+                 Prev -> o - ln reverse f
+                 Next -> o + ln id ss
   modify $ \s -> s { offset = newoff }
 
-moveHistory :: Direction -> XP ()
-moveHistory d = do
-  h <- getHistory
-  c <- gets command
-  let str = if h /= [] then head h else c
-  let nc = case elemIndex c h of
-             Just i -> case d of
-                         Prev -> h !! (if (i + 1) > (length h - 1) then 0 else i + 1)
-                         Next -> h !! (max (i - 1) 0)
-             Nothing -> str
-  modify $ \s -> s { command = nc, offset = length nc}
+moveHistory :: (W.Stack String -> W.Stack String) -> XP ()
+moveHistory f = modify $ \s -> let ch = f $ commandHistory s
+                               in s { commandHistory = ch
+                                    , offset         = length $ W.focus ch }
 
 -- X Stuff
 
@@ -550,7 +626,7 @@ printPrompt drw = do
 getCompletions :: XP [String]
 getCompletions = do
   s <- get
-  io $ (completionFunction s) (commandToComplete (xptype s) (command s))
+  io $ completionFunction s (commandToComplete (xptype s) (command s))
        `catch` \_ -> return []
 
 setComplWin :: Window -> ComplWindowDim -> XP ()
@@ -590,9 +666,9 @@ getComplWinDim compl = do
 
   tws <- mapM (textWidthXMF (dpy st) fs) compl
   let max_compl_len =  fromIntegral ((fi ht `div` 2) + maximum tws)
-      columns = max 1 $ wh `div` (fi max_compl_len)
+      columns = max 1 $ wh `div` fi max_compl_len
       rem_height =  rect_height scr - ht
-      (rows,r) = (length compl) `divMod` fi columns
+      (rows,r) = length compl `divMod` fi columns
       needed_rows = max 1 (rows + if r == 0 then 0 else 1)
       actual_max_number_of_rows = rem_height `div` ht
       actual_rows = min actual_max_number_of_rows (fi needed_rows)
@@ -624,7 +700,7 @@ drawComplWin w compl = do
   p <- io $ createPixmap d w wh ht
                          (defaultDepthOfScreen scr)
   io $ fillDrawable d p gc border bgcolor (fi bw) wh ht
-  let ac = splitInSubListsAt (length yy) (take ((length xx) * (length yy)) compl)
+  let ac = splitInSubListsAt (length yy) (take (length xx * length yy) compl)
   printComplList d p gc (fgColor c) (bgColor c) xx yy ac
   io $ copyArea d p w gc 0 0 wh ht 0 0
   io $ freePixmap d p
@@ -636,7 +712,7 @@ redrawComplWin compl = do
   let recreate = do destroyComplWin
                     w <- createComplWin nwi
                     drawComplWin w compl
-  if (compl /= [] )
+  if compl /= [] && showComplWin st
      then case complWin st of
             Just w -> case complWinDim st of
                         Just wi -> if nwi == wi -- complWinDim did not change
@@ -673,41 +749,28 @@ printComplString d drw gc fc bc x y s = do
 
 -- History
 
-data History =
-    H { prompt :: String
-      , command_history :: String
-      } deriving (Show, Read, Eq)
+type History = M.Map String [String]
 
-historyPush :: XP ()
-historyPush = do
-  c <- gets command
-  when (c /= []) $ modify (\s -> s { history = nub $ H (showXPrompt (xptype s)) c : history s })
+emptyHistory :: History
+emptyHistory = M.empty
 
-getHistory :: XP [String]
-getHistory = do
-  hist <- gets history
-  pt <- gets xptype
-  return $ map command_history . filter (\h -> prompt h == showXPrompt pt) $ hist
+getHistoryFile :: IO FilePath
+getHistoryFile = fmap (++ "/history") $ getAppUserDataDirectory "xmonad"
 
-readHistory :: IO ([History],Handle)
-readHistory = do
-  home <- getEnv "HOME"
-  let path = home ++ "/.xmonad/history"
-  f <- fileExist path
-  if f then do h <- openFile path ReadMode
-               str <- hGetContents h
-               case (reads str) of
-                 [(hist,_)] -> return (hist,h)
-                 [] -> return ([],h)
-                 _ -> return ([],h)
-       else do h <- openFile path WriteMode
-               return ([],h)
+readHistory :: IO History
+readHistory = catch readHist (const (return emptyHistory))
+ where
+    readHist = do
+        path <- getHistoryFile
+        xs <- bracket (openFile path ReadMode) hClose hGetLine
+        readIO xs
 
-writeHistory :: [History] -> IO ()
+writeHistory :: History -> IO ()
 writeHistory hist = do
-  home <- getEnv "HOME"
-  let path = home ++ "/.xmonad/history"
-  catch (writeFile path (show hist)) (\_ -> do putStrLn "error in writing"; return ())
+  path <- getHistoryFile
+  catch (writeFile path (show hist)) $ const $ hPutStrLn stderr "error in writing"
+  setFileMode path mode
+    where mode = ownerReadMode .|. ownerWriteMode
 
 -- $xutils
 
@@ -802,18 +865,27 @@ breakAtSpace s
       where (s1, s2 ) = break isSpace s
             (s1',s2') = breakAtSpace $ tail s2
 
--- | Sort a list and remove duplicates.
+-- | 'historyCompletion' provides a canned completion function much like
+--   'getShellCompl'; you pass it to mkXPrompt, and it will make completions work
+--   from the query history stored in ~\/.xmonad\/history.
+historyCompletion :: ComplFunction
+historyCompletion = historyCompletionP (const True)
+
+-- | Like 'historyCompletion' but only uses history data from Prompts whose
+-- name satisfies the given predicate.
+historyCompletionP :: (String -> Bool) -> ComplFunction
+historyCompletionP p x = fmap (toComplList . M.filterWithKey (const . p)) readHistory
+    where toComplList = deleteConsecutive . filter (isInfixOf x) . M.fold (++) []
+
+-- | Sort a list and remove duplicates. Like 'deleteAllDuplicates', but trades off
+--   laziness and stability for efficiency.
 uniqSort :: Ord a => [a] -> [a]
 uniqSort = toList . fromList
 
--- | 'historyCompletion' provides a canned completion function much like
---   getShellCompl; you pass it to mkXPrompt, and it will make completions work
---   from the query history stored in ~/.xmonad/history.
-historyCompletion :: ComplFunction
-historyCompletion = \x -> liftM (filter $ isInfixOf x) readHistoryIO
-
--- We need to define this locally because there is no function with the type "XP a -> IO a", and
--- 'getHistory' is uselessly of the type "XP [String]".
-readHistoryIO :: IO [String]
-readHistoryIO = do (hist,_) <- readHistory
-                   return $ map command_history hist
\ No newline at end of file
+-- | Functions to be used with the 'historyFilter' setting.
+-- 'deleteAllDuplicates' will remove all duplicate entries.
+-- 'deleteConsecutive' will only remove duplicate elements
+-- immediately next to each other.
+deleteAllDuplicates, deleteConsecutive :: [String] -> [String]
+deleteAllDuplicates = nub
+deleteConsecutive = map head . group
