@@ -15,7 +15,8 @@
 module XMonad.Prompt.Shell
     ( -- * Usage
       -- $usage
-      shellPrompt
+      Shell (..)
+    , shellPrompt
     , getCommands
     , getBrowser
     , getEditor
@@ -43,7 +44,7 @@ import XMonad.Prompt
 --
 -- 2. In your keybindings add something like:
 --
--- >   , ((modMask x .|. controlMask, xK_x), shellPrompt defaultXPConfig)
+-- >   , ((modm .|. controlMask, xK_x), shellPrompt defaultXPConfig)
 --
 -- For detailed instruction on editing the key binding see
 -- "XMonad.Doc.Extending#Editing_key_bindings".
@@ -56,7 +57,7 @@ instance XPrompt Shell where
 
 shellPrompt :: XPConfig -> X ()
 shellPrompt c = do
-    cmds <- io $ getCommands
+    cmds <- io getCommands
     mkXPrompt Shell c (getShellCompl cmds) (spawn . encodeOutput)
 
 -- | See safe and unsafeSpawn. prompt is an alias for safePrompt;
@@ -66,8 +67,8 @@ shellPrompt c = do
 -- is the program to be run with the interactive input.
 -- You would use these like this:
 --
--- >     , ((modMask,               xK_b), safePrompt "firefox" greenXPConfig)
--- >     , ((modMask .|. shiftMask, xK_c), prompt ("xterm" ++ " -e") greenXPConfig)
+-- >     , ((modm,               xK_b), safePrompt "firefox" greenXPConfig)
+-- >     , ((modm .|. shiftMask, xK_c), prompt ("xterm" ++ " -e") greenXPConfig)
 --
 -- Note that you want to use safePrompt for Firefox input, as Firefox
 -- wants URLs, and unsafePrompt for the XTerm example because this allows
@@ -76,7 +77,7 @@ shellPrompt c = do
 prompt, unsafePrompt, safePrompt :: FilePath -> XPConfig -> X ()
 prompt = unsafePrompt
 safePrompt c config = mkXPrompt Shell config (getShellCompl [c]) run
-    where run = safeSpawn c . encodeOutput
+    where run = safeSpawn c . return . encodeOutput
 unsafePrompt c config = mkXPrompt Shell config (getShellCompl [c]) run
     where run a = unsafeSpawn $ c ++ " " ++ encodeOutput a
 
@@ -117,13 +118,12 @@ split e l =
 
 escape :: String -> String
 escape []       = ""
-escape (' ':xs) = "\\ " ++ escape xs
 escape (x:xs)
     | isSpecialChar x = '\\' : x : escape xs
     | otherwise       = x : escape xs
 
 isSpecialChar :: Char -> Bool
-isSpecialChar =  flip elem "\\@\"'#?$*()[]{};"
+isSpecialChar =  flip elem " &\\@\"'#?$*()[]{};"
 
 -- | Ask the shell environment for
 env :: String -> String -> IO String
@@ -141,4 +141,4 @@ getBrowser = env "BROWSER" "firefox"
 
 -- | Like 'getBrowser', but should be of a text editor. This gets the $EDITOR variable, defaulting to \"emacs\".
 getEditor :: IO String
-getEditor = env "EDITOR" "emacs"
\ No newline at end of file
+getEditor = env "EDITOR" "emacs"
