@@ -37,7 +37,7 @@ import Data.Maybe
 --
 -- 2. In your keybindings add something like:
 --
--- >   , ((modMask x .|. controlMask, xK_s), sshPrompt defaultXPConfig)
+-- >   , ((modm .|. controlMask, xK_s), sshPrompt defaultXPConfig)
 --
 -- Keep in mind, that if you want to use the completion you have to
 -- disable the "HashKnownHosts" option in your ssh_config
@@ -54,11 +54,11 @@ instance XPrompt Ssh where
 
 sshPrompt :: XPConfig -> X ()
 sshPrompt c = do
-  sc <- io $ sshComplList
+  sc <- io sshComplList
   mkXPrompt Ssh c (mkComplFunFromList sc) ssh
 
 ssh :: String -> X ()
-ssh s = runInTerm "" ("ssh " ++ s)
+ssh = runInTerm "" . ("ssh " ++ )
 
 sshComplList :: IO [String]
 sshComplList = uniqSort `fmap` liftM2 (++) sshComplListLocal sshComplListGlobal
