@@ -4,7 +4,7 @@
 -- Copyright   :  (C) 2007 Andrea Rossato, David Roundy
 -- License     :  BSD3
 --
--- Maintainer  :  droundy@darcs.net
+-- Maintainer  :  
 -- Stability   :  unstable
 -- Portability :  unportable
 --
@@ -29,7 +29,7 @@ import XMonad.Util.WorkspaceCompare ( getSortByIndex )
 -- > import XMonad.Prompt
 -- > import XMonad.Prompt.Workspace
 --
--- >   , ((modMask x .|. shiftMask, xK_m     ), workspacePrompt defaultXPConfig (windows . W.shift))
+-- >   , ((modm .|. shiftMask, xK_m     ), workspacePrompt defaultXPConfig (windows . W.shift))
 --
 -- For detailed instruction on editing the key binding see
 -- "XMonad.Doc.Extending#Editing_key_bindings".
@@ -43,7 +43,4 @@ workspacePrompt :: XPConfig -> (String -> X ()) -> X ()
 workspacePrompt c job = do ws <- gets (workspaces . windowset)
                            sort <- getSortByIndex
                            let ts = map tag $ sort ws
-                           mkXPrompt (Wor "") c (mkCompl ts) job
-
-mkCompl :: [String] -> String -> IO [String]
-mkCompl l s = return $ filter (\x -> take (length s) x == s) l
+                           mkXPrompt (Wor "") c (mkComplFunFromList' ts) job
