@@ -27,13 +27,16 @@ module XMonad.Util.EZConfig (
                              -- * Emacs-style keybinding specifications
 
                              mkKeymap, checkKeymap,
+                             mkNamedKeymap
                             ) where
 
 import XMonad
 import XMonad.Actions.Submap
 
+import XMonad.Util.NamedActions
+
 import qualified Data.Map as M
-import Data.List (foldl', intersperse, sortBy, groupBy, nub)
+import Data.List (foldl', sortBy, groupBy, nub)
 import Data.Ord (comparing)
 import Data.Maybe
 import Control.Arrow (first, (&&&))
@@ -56,7 +59,7 @@ import Text.ParserCombinators.ReadP
 -- provided, suitable for adding to your 'startupHook', which can warn
 -- you of any parse errors or duplicate bindings in your keymap.
 --
--- For more information and usage eamples, see the documentation
+-- For more information and usage examples, see the documentation
 -- provided with each exported function, and check the xmonad config
 -- archive (<http://haskell.org/haskellwiki/Xmonad/Config_archive>)
 -- for some real examples of use.
@@ -134,11 +137,16 @@ removeMouseBindings conf mouseBindingList =
 --   the key sequence descriptions contained in the Strings.  The key
 --   sequence descriptions are \"emacs-style\": @M-@, @C-@, @S-@, and
 --   @M\#-@ denote mod, control, shift, and mod1-mod5 (where @\#@ is
---   replaced by the appropriate number) respectively; some special
---   keys can be specified by enclosing their name in angle brackets.
+--   replaced by the appropriate number) respectively.  Note that if
+--   you want to make a keybinding using \'alt\' even though you use a
+--   different key (like the \'windows\' key) for \'mod\', you can use
+--   something like @\"M1-x\"@ for alt+x (check the output of @xmodmap@
+--   to see which mod key \'alt\' is bound to). Some special keys can
+--   also be specified by enclosing their name in angle brackets.
 --
---   For example, @\"M-C-x\"@ denotes mod+ctrl+x; @\"S-\<Escape\>\"@ denotes
---   shift-escape.
+--   For example, @\"M-C-x\"@ denotes mod+ctrl+x; @\"S-\<Escape\>\"@
+--   denotes shift-escape; @\"M1-C-\<Delete\>\"@ denotes alt+ctrl+delete
+--   (assuming alt is bound to mod1, which is common).
 --
 --   Sequences of keys can also be specified by separating the key
 --   descriptions with spaces. For example, @\"M-x y \<Down\>\"@ denotes the
@@ -159,7 +167,10 @@ removeMouseBindings conf mouseBindingList =
 -- create a keymap and add it to your config.
 --
 -- Here is a complete list of supported special keys.  Note that a few
--- keys, such as the arrow keys, have synonyms:
+-- keys, such as the arrow keys, have synonyms.  If there are other
+-- special keys you would like to see supported, feel free to submit a
+-- patch, or ask on the xmonad mailing list; adding special keys is
+-- quite simple.
 --
 -- > <Backspace>
 -- > <Tab>
@@ -167,6 +178,7 @@ removeMouseBindings conf mouseBindingList =
 -- > <Pause>
 -- > <Scroll_lock>
 -- > <Sys_Req>
+-- > <Print>
 -- > <Escape>, <Esc>
 -- > <Delete>
 -- > <Home>
@@ -208,30 +220,158 @@ removeMouseBindings conf mouseBindingList =
 -- > <KP_Subtract>
 -- > <KP_Decimal>
 -- > <KP_Divide>
--- > <KP_0>
--- > <KP_1>
--- > <KP_2>
--- > <KP_3>
--- > <KP_4>
--- > <KP_5>
--- > <KP_6>
--- > <KP_7>
--- > <KP_8>
--- > <KP_9>
+-- > <KP_0>-<KP_9>
+--
+-- Long list of multimedia keys. Please note that not all keys may be
+-- present in your particular setup although most likely they will do.
+--
+-- > <XF86ModeLock>
+-- > <XF86MonBrightnessUp>
+-- > <XF86MonBrightnessDown>
+-- > <XF86KbdLightOnOff>
+-- > <XF86KbdBrightnessUp>
+-- > <XF86KbdBrightnessDown>
+-- > <XF86Standby>
+-- > <XF86AudioLowerVolume>
+-- > <XF86AudioMute>
+-- > <XF86AudioRaiseVolume>
+-- > <XF86AudioPlay>
+-- > <XF86AudioStop>
+-- > <XF86AudioPrev>
+-- > <XF86AudioNext>
+-- > <XF86HomePage>
+-- > <XF86Mail>
+-- > <XF86Start>
+-- > <XF86Search>
+-- > <XF86AudioRecord>
+-- > <XF86Calculator>
+-- > <XF86Memo>
+-- > <XF86ToDoList>
+-- > <XF86Calendar>
+-- > <XF86PowerDown>
+-- > <XF86ContrastAdjust>
+-- > <XF86RockerUp>
+-- > <XF86RockerDown>
+-- > <XF86RockerEnter>
+-- > <XF86Back>
+-- > <XF86Forward>
+-- > <XF86Stop>
+-- > <XF86Refresh>
+-- > <XF86PowerOff>
+-- > <XF86WakeUp>
+-- > <XF86Eject>
+-- > <XF86ScreenSaver>
+-- > <XF86WWW>
+-- > <XF86Sleep>
+-- > <XF86Favorites>
+-- > <XF86AudioPause>
+-- > <XF86AudioMedia>
+-- > <XF86MyComputer>
+-- > <XF86VendorHome>
+-- > <XF86LightBulb>
+-- > <XF86Shop>
+-- > <XF86History>
+-- > <XF86OpenURL>
+-- > <XF86AddFavorite>
+-- > <XF86HotLinks>
+-- > <XF86BrightnessAdjust>
+-- > <XF86Finance>
+-- > <XF86Community>
+-- > <XF86AudioRewind>
+-- > <XF86XF86BackForward>
+-- > <XF86Launch0>-<XF86Launch9>, <XF86LaunchA>-<XF86LaunchF>
+-- > <XF86ApplicationLeft>
+-- > <XF86ApplicationRight>
+-- > <XF86Book>
+-- > <XF86CD>
+-- > <XF86Calculater>
+-- > <XF86Clear>
+-- > <XF86Close>
+-- > <XF86Copy>
+-- > <XF86Cut>
+-- > <XF86Display>
+-- > <XF86DOS>
+-- > <XF86Documents>
+-- > <XF86Excel>
+-- > <XF86Explorer>
+-- > <XF86Game>
+-- > <XF86Go>
+-- > <XF86iTouch>
+-- > <XF86LogOff>
+-- > <XF86Market>
+-- > <XF86Meeting>
+-- > <XF86MenuKB>
+-- > <XF86MenuPB>
+-- > <XF86MySites>
+-- > <XF86New>
+-- > <XF86News>
+-- > <XF86OfficeHome>
+-- > <XF86Open>
+-- > <XF86Option>
+-- > <XF86Paste>
+-- > <XF86Phone>
+-- > <XF86Q>
+-- > <XF86Reply>
+-- > <XF86Reload>
+-- > <XF86RotateWindows>
+-- > <XF86RotationPB>
+-- > <XF86RotationKB>
+-- > <XF86Save>
+-- > <XF86ScrollUp>
+-- > <XF86ScrollDown>
+-- > <XF86ScrollClick>
+-- > <XF86Send>
+-- > <XF86Spell>
+-- > <XF86SplitScreen>
+-- > <XF86Support>
+-- > <XF86TaskPane>
+-- > <XF86Terminal>
+-- > <XF86Tools>
+-- > <XF86Travel>
+-- > <XF86UserPB>
+-- > <XF86User1KB>
+-- > <XF86User2KB>
+-- > <XF86Video>
+-- > <XF86WheelButton>
+-- > <XF86Word>
+-- > <XF86Xfer>
+-- > <XF86ZoomIn>
+-- > <XF86ZoomOut>
+-- > <XF86Away>
+-- > <XF86Messenger>
+-- > <XF86WebCam>
+-- > <XF86MailForward>
+-- > <XF86Pictures>
+-- > <XF86Music>
+-- > <XF86_Switch_VT_1>-<XF86_Switch_VT_12>
+-- > <XF86_Ungrab>
+-- > <XF86_ClearGrab>
+-- > <XF86_Next_VMode>
+-- > <XF86_Prev_VMode>
 
 mkKeymap :: XConfig l -> [(String, X ())] -> M.Map (KeyMask, KeySym) (X ())
 mkKeymap c = M.fromList . mkSubmaps . readKeymap c
 
+mkNamedKeymap :: XConfig l -> [(String, NamedAction)] -> [((KeyMask, KeySym), NamedAction)]
+mkNamedKeymap c = mkNamedSubmaps . readKeymap c
+
 -- | Given a list of pairs of parsed key sequences and actions,
 --   group them into submaps in the appropriate way.
+
+mkNamedSubmaps :: [([(KeyMask, KeySym)], NamedAction)] -> [((KeyMask, KeySym), NamedAction)]
+mkNamedSubmaps = mkSubmaps' submapName
+
 mkSubmaps :: [ ([(KeyMask,KeySym)], X ()) ] -> [((KeyMask, KeySym), X ())]
-mkSubmaps binds = map combine gathered
+mkSubmaps = mkSubmaps' $ submap . M.fromList
+
+mkSubmaps' ::  (Ord a) => ([(a, c)] -> c) -> [([a], c)] -> [(a, c)]
+mkSubmaps' subm binds = map combine gathered
   where gathered = groupBy fstKey
                  . sortBy (comparing fst)
                  $ binds
         combine [([k],act)] = (k,act)
         combine ks = (head . fst . head $ ks,
-                      submap . M.fromList . mkSubmaps $ map (first tail) ks)
+                      subm . mkSubmaps' subm $ map (first tail) ks)
         fstKey = (==) `on` (head . fst)
 
 on :: (a -> a -> b) -> (c -> a) -> c -> c -> b
@@ -241,7 +381,7 @@ op `on` f = \x y -> f x `op` f y
 --   description, action) pairs, parse the key sequences into lists of
 --   @(KeyMask,KeySym)@ pairs.  Key sequences which fail to parse will
 --   be ignored.
-readKeymap :: XConfig l -> [(String, X())] -> [([(KeyMask,KeySym)], X())]
+readKeymap :: XConfig l -> [(String, t)] -> [([(KeyMask, KeySym)], t)]
 readKeymap c = mapMaybe (maybeKeys . first (readKeySequence c))
   where maybeKeys (Nothing,_) = Nothing
         maybeKeys (Just k, act) = Just (k, act)
@@ -273,7 +413,8 @@ parseModifier c =  (string "M-" >> return (modMask c))
                +++ do char 'M'
                       n <- satisfy (`elem` ['1'..'5'])
                       char '-'
-                      return (mod1Mask + (read [n]) - 1)
+                      return $ indexMod (read [n] - 1)
+    where indexMod = (!!) [mod1Mask,mod2Mask,mod3Mask,mod4Mask,mod5Mask]
 
 -- | Parse an unmodified basic key, like @\"x\"@, @\"<F1>\"@, etc.
 parseKey :: ReadP KeySym
@@ -296,12 +437,12 @@ parseSpecial = do char '<'
 
 -- | A list of all special key names and their associated KeySyms.
 keyNames :: [(String, KeySym)]
-keyNames = functionKeys ++ specialKeys
+keyNames = functionKeys ++ specialKeys ++ multimediaKeys
 
 -- | A list pairing function key descriptor strings (e.g. @\"<F2>\"@) with
 --   the associated KeySyms.
 functionKeys :: [(String, KeySym)]
-functionKeys = [ ("F" ++ show n, k)
+functionKeys = [ ('F' : show n, k)
                | (n,k) <- zip ([1..24] :: [Int]) [xK_F1..] ]
 
 -- | A list of special key names and their corresponding KeySyms.
@@ -312,6 +453,7 @@ specialKeys = [ ("Backspace"  , xK_BackSpace)
               , ("Pause"      , xK_Pause)
               , ("Scroll_lock", xK_Scroll_Lock)
               , ("Sys_Req"    , xK_Sys_Req)
+              , ("Print"      , xK_Print)
               , ("Escape"     , xK_Escape)
               , ("Esc"        , xK_Escape)
               , ("Delete"     , xK_Delete)
@@ -369,6 +511,160 @@ specialKeys = [ ("Backspace"  , xK_BackSpace)
               , ("KP_9"       , xK_KP_9)
               ]
 
+-- | List of multimedia keys. If X server does not know about some
+-- | keysym it's omitted from list. (stringToKeysym returns noSymbol in this case)
+multimediaKeys :: [(String, KeySym)]
+multimediaKeys = filter ((/= noSymbol) . snd) . map (id &&& stringToKeysym) $
+                 [ "XF86ModeLock"
+                 , "XF86MonBrightnessUp"
+                 , "XF86MonBrightnessDown"
+                 , "XF86KbdLightOnOff"
+                 , "XF86KbdBrightnessUp"
+                 , "XF86KbdBrightnessDown"
+                 , "XF86Standby"
+                 , "XF86AudioLowerVolume"
+                 , "XF86AudioMute"
+                 , "XF86AudioRaiseVolume"
+                 , "XF86AudioPlay"
+                 , "XF86AudioStop"
+                 , "XF86AudioPrev"
+                 , "XF86AudioNext"
+                 , "XF86HomePage"
+                 , "XF86Mail"
+                 , "XF86Start"
+                 , "XF86Search"
+                 , "XF86AudioRecord"
+                 , "XF86Calculator"
+                 , "XF86Memo"
+                 , "XF86ToDoList"
+                 , "XF86Calendar"
+                 , "XF86PowerDown"
+                 , "XF86ContrastAdjust"
+                 , "XF86RockerUp"
+                 , "XF86RockerDown"
+                 , "XF86RockerEnter"
+                 , "XF86Back"
+                 , "XF86Forward"
+                 , "XF86Stop"
+                 , "XF86Refresh"
+                 , "XF86PowerOff"
+                 , "XF86WakeUp"
+                 , "XF86Eject"
+                 , "XF86ScreenSaver"
+                 , "XF86WWW"
+                 , "XF86Sleep"
+                 , "XF86Favorites"
+                 , "XF86AudioPause"
+                 , "XF86AudioMedia"
+                 , "XF86MyComputer"
+                 , "XF86VendorHome"
+                 , "XF86LightBulb"
+                 , "XF86Shop"
+                 , "XF86History"
+                 , "XF86OpenURL"
+                 , "XF86AddFavorite"
+                 , "XF86HotLinks"
+                 , "XF86BrightnessAdjust"
+                 , "XF86Finance"
+                 , "XF86Community"
+                 , "XF86AudioRewind"
+                 , "XF86BackForward"
+                 , "XF86Launch0"
+                 , "XF86Launch1"
+                 , "XF86Launch2"
+                 , "XF86Launch3"
+                 , "XF86Launch4"
+                 , "XF86Launch5"
+                 , "XF86Launch6"
+                 , "XF86Launch7"
+                 , "XF86Launch8"
+                 , "XF86Launch9"
+                 , "XF86LaunchA"
+                 , "XF86LaunchB"
+                 , "XF86LaunchC"
+                 , "XF86LaunchD"
+                 , "XF86LaunchE"
+                 , "XF86LaunchF"
+                 , "XF86ApplicationLeft"
+                 , "XF86ApplicationRight"
+                 , "XF86Book"
+                 , "XF86CD"
+                 , "XF86Calculater"
+                 , "XF86Clear"
+                 , "XF86Close"
+                 , "XF86Copy"
+                 , "XF86Cut"
+                 , "XF86Display"
+                 , "XF86DOS"
+                 , "XF86Documents"
+                 , "XF86Excel"
+                 , "XF86Explorer"
+                 , "XF86Game"
+                 , "XF86Go"
+                 , "XF86iTouch"
+                 , "XF86LogOff"
+                 , "XF86Market"
+                 , "XF86Meeting"
+                 , "XF86MenuKB"
+                 , "XF86MenuPB"
+                 , "XF86MySites"
+                 , "XF86New"
+                 , "XF86News"
+                 , "XF86OfficeHome"
+                 , "XF86Open"
+                 , "XF86Option"
+                 , "XF86Paste"
+                 , "XF86Phone"
+                 , "XF86Q"
+                 , "XF86Reply"
+                 , "XF86Reload"
+                 , "XF86RotateWindows"
+                 , "XF86RotationPB"
+                 , "XF86RotationKB"
+                 , "XF86Save"
+                 , "XF86ScrollUp"
+                 , "XF86ScrollDown"
+                 , "XF86ScrollClick"
+                 , "XF86Send"
+                 , "XF86Spell"
+                 , "XF86SplitScreen"
+                 , "XF86Support"
+                 , "XF86TaskPane"
+                 , "XF86Terminal"
+                 , "XF86Tools"
+                 , "XF86Travel"
+                 , "XF86UserPB"
+                 , "XF86User1KB"
+                 , "XF86User2KB"
+                 , "XF86Video"
+                 , "XF86WheelButton"
+                 , "XF86Word"
+                 , "XF86Xfer"
+                 , "XF86ZoomIn"
+                 , "XF86ZoomOut"
+                 , "XF86Away"
+                 , "XF86Messenger"
+                 , "XF86WebCam"
+                 , "XF86MailForward"
+                 , "XF86Pictures"
+                 , "XF86Music"
+                 , "XF86_Switch_VT_1"
+                 , "XF86_Switch_VT_2"
+                 , "XF86_Switch_VT_3"
+                 , "XF86_Switch_VT_4"
+                 , "XF86_Switch_VT_5"
+                 , "XF86_Switch_VT_6"
+                 , "XF86_Switch_VT_7"
+                 , "XF86_Switch_VT_8"
+                 , "XF86_Switch_VT_9"
+                 , "XF86_Switch_VT_10"
+                 , "XF86_Switch_VT_11"
+                 , "XF86_Switch_VT_12"
+                 , "XF86_Ungrab"
+                 , "XF86_ClearGrab"
+                 , "XF86_Next_VMode"
+                 , "XF86_Prev_VMode" ]
+
 -- | Given a configuration record and a list of (key sequence
 --   description, action) pairs, check the key sequence descriptions
 --   for validity, and warn the user (via a popup xmessage window) of
@@ -408,7 +704,7 @@ checkKeymap conf km = warn (doKeymapCheck conf km)
                             ++ msg "duplicate" dup ++ "'"
         msg _ [] = ""
         msg m xs = m ++ " keybindings detected: " ++ showBindings xs
-        showBindings = concat . intersperse " " . map ((++"\"") . ("\""++))
+        showBindings = unwords . map (("\""++) . (++"\""))
 
 -- | Given a config and a list of (key sequence description, action)
 --   pairs, check the key sequence descriptions for validity,
