@@ -21,6 +21,7 @@ module XMonad.Util.Run (
                           runProcessWithInput,
                           runProcessWithInputAndWait,
                           safeSpawn,
+                          safeSpawnProg,
                           unsafeSpawn,
                           runInTerm,
                           safeRunInTerm,
@@ -31,11 +32,11 @@ module XMonad.Util.Run (
                          ) where
 
 import System.Posix.IO
-import System.Posix.Process (executeFile)
+import System.Posix.Process (executeFile, forkProcess, createSession)
 import Control.Concurrent (threadDelay)
-import Control.Exception (try)
+import Control.Exception (try) -- use OldException with base 4
 import System.IO
-import System.Process (runInteractiveProcess, waitForProcess)
+import System.Process (runInteractiveProcess)
 import XMonad
 import Control.Monad
 
@@ -50,33 +51,33 @@ import Control.Monad
 -- For an example usage of 'runProcessWithInputAndWait' see
 -- "XMonad.Util.Dzen"
 
--- | Return output if the command succeeded, otherwise return @()@.
--- This corresponds to dmenu's notion of exit code 1 for a cancelled invocation.
+-- | Returns the output.
 runProcessWithInput :: FilePath -> [String] -> String -> IO String
 runProcessWithInput cmd args input = do
-    (pin, pout, perr, ph) <- runInteractiveProcess cmd args Nothing Nothing
+    (pin, pout, perr, _) <- runInteractiveProcess cmd args Nothing Nothing
     hPutStr pin input
     hClose pin
     output <- hGetContents pout
     when (output == output) $ return ()
     hClose pout
     hClose perr
-    waitForProcess ph
+    -- no need to waitForProcess, we ignore SIGCHLD
     return output
 
 -- | Wait is in µs (microseconds)
 runProcessWithInputAndWait :: FilePath -> [String] -> String -> Int -> IO ()
 runProcessWithInputAndWait cmd args input timeout = do
-    doubleFork $ do
-        (pin, pout, perr, ph) <- runInteractiveProcess cmd args Nothing Nothing
+    forkProcess $ do
+        (pin, pout, perr, _) <- runInteractiveProcess cmd args Nothing Nothing
         hPutStr pin input
         hFlush pin
         threadDelay timeout
         hClose pin
         hClose pout
         hClose perr
-        waitForProcess ph
+        -- no need to waitForProcess, we ignore SIGCHLD
         return ()
+    return ()
 
 -- | Multiplies by ONE MILLION, for functions that take microseconds.
 --
@@ -98,15 +99,21 @@ unsafeSpawn is internally an alias for XMonad's 'spawn', to remind one that use
 of it can be, well, unsafe.
 Examples:
 
-> , ((modMask, xK_Print), unsafeSpawn "import -window root $HOME/xwd-$(date +%s)$$.png")
-> , ((modMask, xK_d    ), safeSpawn "firefox" "")
+> , ((modm, xK_Print), unsafeSpawn "import -window root $HOME/xwd-$(date +%s)$$.png")
+> , ((modm, xK_d    ), safeSpawn "firefox" "")
 
 Note that the unsafeSpawn example must be unsafe and not safe because
 it makes use of shell interpretation by relying on @$HOME@ and
 interpolation, whereas the safeSpawn example can be safe because
 Firefox doesn't need any arguments if it is just being started. -}
-safeSpawn :: MonadIO m => FilePath -> String -> m ()
-safeSpawn prog arg = liftIO (try (doubleFork $ executeFile prog True [arg] Nothing) >> return ())
+safeSpawn :: MonadIO m => FilePath -> [String] -> m ()
+safeSpawn prog args = liftIO (try (forkProcess $ executeFile prog True args Nothing) >> return ())
+
+-- | Like 'safeSpawn', but only takes a program (and no arguments for it). eg.
+--
+-- > safeSpawnProg "firefox"
+safeSpawnProg :: MonadIO m => FilePath -> m ()
+safeSpawnProg = flip safeSpawn []
 
 unsafeSpawn :: MonadIO m => String -> m ()
 unsafeSpawn = spawn
@@ -119,7 +126,7 @@ runInTerm = unsafeRunInTerm
 
 -- | Run a given program in the preferred terminal emulator; see 'runInTerm'. This makes use of 'safeSpawn'.
 safeRunInTerm :: String -> String -> X ()
-safeRunInTerm options command = asks (terminal . config) >>= \t -> safeSpawn t (options ++ " -e " ++ command)
+safeRunInTerm options command = asks (terminal . config) >>= \t -> safeSpawn t [options, " -e " ++ command]
 
 -- | Launch an external application through the system shell and return a @Handle@ to its standard input.
 spawnPipe :: String -> IO Handle
@@ -128,7 +135,10 @@ spawnPipe x = do
     setFdOption wr CloseOnExec True
     h <- fdToHandle wr
     hSetBuffering h LineBuffering
-    doubleFork $ do
+    forkProcess $ do
+        createSession
+        uninstallSignalHandlers
         dupTo rd stdInput
         executeFile "/bin/sh" False ["-c", x] Nothing
+    closeFd rd
     return h
