@@ -8,19 +8,26 @@
 -- Stability   :  unstable
 -- Portability :  unportable
 --
--- EDSL for specifying window properties, such as title, classname or resource.
+-- EDSL for specifying window properties; various utilities related to window
+-- properties.
 --
 -----------------------------------------------------------------------------
 module XMonad.Util.WindowProperties (
-    -- * Usage
-    -- $usage
-    Property(..), hasProperty, focusedHasProperty)
+    -- * EDSL for window properties
+    -- $edsl
+    Property(..), hasProperty, focusedHasProperty, allWithProperty,
+    propertyToQuery,
+    -- * Helper functions
+    -- $helpers
+    getProp32, getProp32s)
 where
 import XMonad
 import qualified XMonad.StackSet as W
+import Foreign.C.Types (CLong)
+import Control.Monad
 
--- $usage
--- This module allows to specify window properties, such as title, classname or
+-- $edsl
+-- Allows to specify window properties, such as title, classname or
 -- resource, and to check them.
 --
 -- In contrast to ManageHook properties, these are instances of Show and Read,
@@ -31,7 +38,8 @@ data Property = Title String
               | ClassName String
               | Resource String
               | Role String -- ^ WM_WINDOW_ROLE property
-              | And Property Property  
+              | Machine String -- ^ WM_CLIENT_MACHINE property
+              | And Property Property
               | Or  Property Property
               | Not Property
               | Const Bool
@@ -44,7 +52,8 @@ hasProperty :: Property -> Window -> X Bool
 hasProperty (Title s)     w = withDisplay $ \d -> fmap (Just s ==) $ io $ fetchName d w
 hasProperty (Resource s)  w = withDisplay $ \d -> fmap ((==) s . resName ) $ io $ getClassHint d w
 hasProperty (ClassName s) w = withDisplay $ \d -> fmap ((==) s . resClass) $ io $ getClassHint d w
-hasProperty (Role s) w = withDisplay $ \d -> fmap ((==) (Just s)) $ getStringProperty d w "WM_WINDOW_ROLE"
+hasProperty (Role s)      w = withDisplay $ \d -> fmap ((==) (Just s)) $ getStringProperty d w "WM_WINDOW_ROLE"
+hasProperty (Machine s)   w = withDisplay $ \d -> fmap ((==) (Just s)) $ getStringProperty d w "WM_CLIENT_MACHINE"
 hasProperty (And p1 p2)   w = do { r1 <- hasProperty p1 w; r2 <- hasProperty p2 w; return $ r1 && r2 }
 hasProperty (Or p1 p2)    w = do { r1 <- hasProperty p1 w; r2 <- hasProperty p2 w; return $ r1 || r2 }
 hasProperty (Not p1)      w = do { r1 <- hasProperty p1 w; return $ not r1 }
@@ -59,3 +68,31 @@ focusedHasProperty p = do
         Just s  -> hasProperty p $ W.focus s
         Nothing -> return False
 
+-- | Find all existing windows with specified property
+allWithProperty :: Property -> X [Window]
+allWithProperty prop = withDisplay $ \dpy -> do
+    rootw <- asks theRoot
+    (_,_,wins) <- io $ queryTree dpy rootw
+    hasProperty prop `filterM` wins
+
+-- | Convert property to 'Query' 'Bool' (see "XMonad.ManageHook")
+propertyToQuery :: Property -> Query Bool
+propertyToQuery (Title s) = title =? s
+propertyToQuery (Resource s) = resource =? s
+propertyToQuery (ClassName s) = className =? s
+propertyToQuery (Role s) = stringProperty "WM_WINDOW_ROLE" =? s
+propertyToQuery (Machine s) = stringProperty "WM_CLIENT_MACHINE" =? s
+propertyToQuery (And p1 p2) = propertyToQuery p1 <&&> propertyToQuery p2
+propertyToQuery (Or p1 p2) = propertyToQuery p1 <||> propertyToQuery p2
+propertyToQuery (Not p) = not `fmap` propertyToQuery p
+propertyToQuery (Const b) = return b
+
+-- $helpers
+
+-- | Get a window property from atom
+getProp32 :: Atom -> Window -> X (Maybe [CLong])
+getProp32 a w = withDisplay $ \dpy -> io $ getWindowProperty32 dpy a w
+
+-- | Get a window property from string
+getProp32s :: String -> Window -> X (Maybe [CLong])
+getProp32s str w = do { a <- getAtom str; getProp32 a w }
