@@ -1,10 +1,10 @@
+{-# LANGUAGE CPP #-}
 {- |
 Module      :  XMonad.Util.XSelection
 Copyright   :  (C) 2007 Andrea Rossato, Matthew Sackman
 License     :  BSD3
 
-Maintainer  :  Andrea Rossato <andrea.rossato@unibz.it>,
-               Matthew Sackman <matthew@wellquite.org>
+Maintainer  : Gwern Branwen <gwern0@gmail.com>
 Stability   :  unstable
 Portability :  unportable
 
@@ -19,18 +19,20 @@ module XMonad.Util.XSelection (  -- * Usage
                                  getSelection,
                                  promptSelection,
                                  safePromptSelection,
+                                 transformPromptSelection,
+                                 transformSafePromptSelection,
                                  putSelection) where
 
 import Control.Concurrent (forkIO)
 import Control.Exception as E (catch)
 import Control.Monad(Monad (return, (>>)), Functor(..), liftM, join)
-import Data.Bits (shiftL, (.&.), (.|.))
-import Data.Char (chr, ord)
+import Data.Char (ord)
 import Data.Maybe (fromMaybe)
-import Data.Word (Word8)
 import XMonad
 import XMonad.Util.Run (safeSpawn, unsafeSpawn)
 
+import Codec.Binary.UTF8.String (decode)
+
 {- $usage
    Add @import XMonad.Util.XSelection@ to the top of Config.hs
    Then make use of getSelection or promptSelection as needed; if
@@ -38,7 +40,7 @@ import XMonad.Util.Run (safeSpawn, unsafeSpawn)
    the selection string is an URL you just highlighted), then one could add
    to the xmonad.hs a line like thus:
 
-   > , ((modMask .|. shiftMask, xK_b), promptSelection "firefox")
+   > , ((modm .|. shiftMask, xK_b), promptSelection "firefox")
 
    There are a number of known problems with XSelection:
 
@@ -127,42 +129,12 @@ shell using 'safeSpawn' from "XMonad.Util.Run"; see its documentation for more
 details on the advantages and disadvantages of using safeSpawn. -}
 promptSelection, safePromptSelection, unsafePromptSelection :: String -> X ()
 promptSelection = unsafePromptSelection
-safePromptSelection app = join $ io $ liftM (safeSpawn app) (getSelection)
+safePromptSelection app = join $ io $ liftM (safeSpawn app . return) getSelection
 unsafePromptSelection app = join $ io $ liftM unsafeSpawn $ fmap (\x -> app ++ " " ++ x) getSelection
 
-{- | Decode a UTF8 string packed into a list of Word8 values, directly to
-   String; does not deal with CChar, hence you will want the counter-intuitive @map fromIntegral@
-   UTF-8 decoding for internal use in getSelection.
-
-   This code is copied from Eric Mertens's "utf-string" library <http://code.haskell.org/utf8-string/>
-   (as of version 0.1),\which is BSD-3 licensed like this module.
-   It'd be better to just @import Codec.Binary.UTF8.String (decode)@, but then users of this would need to install it; XMonad has enough
-   dependencies already. -}
-decode :: [Word8] -> String
-decode [    ] = ""
-decode (c:cs)
-  | c < 0x80  = chr (fromEnum c) : decode cs
-  | c < 0xc0  = replacement_character : decode cs
-  | c < 0xe0  = multi_byte 1 0x1f 0x80
-  | c < 0xf0  = multi_byte 2 0xf  0x800
-  | c < 0xf8  = multi_byte 3 0x7  0x10000
-  | c < 0xfc  = multi_byte 4 0x3  0x200000
-  | c < 0xfe  = multi_byte 5 0x1  0x4000000
-  | otherwise = replacement_character : decode cs
-  where
-
-    replacement_character :: Char
-    replacement_character = '\xfffd'
-
-    multi_byte :: Int -> Word8 -> Int -> [Char]
-    multi_byte i mask overlong = aux i cs (fromEnum (c .&. mask))
-      where
-        aux 0 rs acc
-          | overlong <= acc && acc <= 0x10ffff &&
-            (acc < 0xd800 || 0xdfff < acc)     &&
-            (acc < 0xfffe || 0xffff < acc)      = chr acc : decode rs
-          | otherwise = replacement_character : decode rs
-        aux n (r:rs) acc
-          | r .&. 0xc0 == 0x80 = aux (n-1) rs
-                               $ shiftL acc 6 .|. fromEnum (r .&. 0x3f)
-        aux _ rs     _ = replacement_character : decode rs
+{- | A wrapper around 'promptSelection' and its safe variant. They take two parameters, the first is a function that transforms strings, and the second is the application to run. The transformer essentially transforms the selection in X.
+One example is to wrap code, such as a command line action copied out of the browser to be run as @"sudo" ++ cmd@ or @"su - -c \""++ cmd ++"\""@.
+-}
+transformPromptSelection, transformSafePromptSelection :: (String -> String) -> String -> X ()
+transformPromptSelection f app = join $ io $ liftM (safeSpawn app . return) (fmap f getSelection)
+transformSafePromptSelection f app = join $ io $ liftM unsafeSpawn $ fmap (\x -> app ++ " " ++ x) (fmap f getSelection)
